@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,7 +14,7 @@ public class Server : Ice.Application
     public override int run(string[] args)
     {
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Callback.Server");
-        adapter.add(new CallbackI(), Ice.Util.stringToIdentity("callback"));
+        adapter.add(new CallbackI(), communicator().stringToIdentity("callback"));
         adapter.activate();
 	communicator().waitForShutdown();
         return 0;
@@ -24,6 +24,9 @@ public class Server : Ice.Application
     {
         Server app = new Server();
         int status = app.main(args, "config.server");
-        System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
