@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -24,8 +24,8 @@ public class SessionServer : Ice.Application
     public override int run(string[] args)
     {
 	Ice.ObjectAdapter adapter = communicator().createObjectAdapter("SessionServer");
-	adapter.add(new DummyPermissionVerifierI(), Ice.Util.stringToIdentity("verifier"));
-	adapter.add(new SessionManagerI(), Ice.Util.stringToIdentity("sessionmanager"));
+	adapter.add(new DummyPermissionVerifierI(), communicator().stringToIdentity("verifier"));
+	adapter.add(new SessionManagerI(), communicator().stringToIdentity("sessionmanager"));
 	adapter.activate();
 	communicator().waitForShutdown();
 	return 0;
@@ -35,6 +35,9 @@ public class SessionServer : Ice.Application
     {
 	SessionServer app = new SessionServer();
         int status = app.main(args, "config.sessionserver");
-        Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
