@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -15,8 +15,8 @@ public class Server : Ice.Application
     public override int run(string[] args)
     {
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Callback.Server");
-        CallbackSenderI sender = new CallbackSenderI();
-        adapter.add(sender, Ice.Util.stringToIdentity("sender"));
+        CallbackSenderI sender = new CallbackSenderI(communicator());
+        adapter.add(sender, communicator().stringToIdentity("sender"));
         adapter.activate();
 
 	Thread t = new Thread(new ThreadStart(sender.Run));
@@ -38,7 +38,10 @@ public class Server : Ice.Application
     public static void Main(string[] args)
     {
         Server app = new Server();
-        int status = app.main(args, "config");
-        System.Environment.Exit(status);
+        int status = app.main(args, "config.server");
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
