@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,7 +12,7 @@ public class Server : Ice.Application
     public override int run(string[] args)
     {
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Hello");
-        adapter.add(new HelloI(), Ice.Util.stringToIdentity("hello"));
+        adapter.add(new HelloI(), communicator().stringToIdentity("hello"));
         adapter.activate();
         communicator().waitForShutdown();
         return 0;
@@ -21,7 +21,10 @@ public class Server : Ice.Application
     public static void Main(string[] args)
     {
         Server app = new Server();
-        int status = app.main(args, "config");
-        System.Environment.Exit(status);
+        int status = app.main(args, "config.server");
+	if(status != 0)
+	{
+	    System.Environment.Exit(status);
+	}
     }
 }
