@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -20,7 +20,7 @@ public class Server
         {
             communicator = Ice.Util.initialize(ref args);
 	    Ice.ObjectAdapter adapter = communicator.createObjectAdapterWithEndpoints("Hello", "tcp -p 10000");
-	    adapter.add(new HelloI(), Ice.Util.stringToIdentity("hello"));
+	    adapter.add(new HelloI(), communicator.stringToIdentity("hello"));
 	    adapter.activate();
 	    communicator.waitForShutdown();
         }
@@ -43,6 +43,9 @@ public class Server
             }
         }
 
-        System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
