@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,7 +21,7 @@ public class Server : Ice.Application
 	Thread reaperThread = new Thread(new ThreadStart(reaper.run));
 	reaperThread.Start();
 
-	adapter.add(new SessionFactoryI(reaper), Ice.Util.stringToIdentity("SessionFactory"));
+	adapter.add(new SessionFactoryI(reaper), communicator().stringToIdentity("SessionFactory"));
 	adapter.activate();
 	communicator().waitForShutdown();
 
@@ -34,7 +34,10 @@ public class Server : Ice.Application
     public static void Main(string[] args)
     {
         Server app = new Server();
-        int status = app.main(args, "config");
-        System.Environment.Exit(status);
+        int status = app.main(args, "config.server");
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
