@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,7 +13,7 @@ public class Server : Ice.Application
     {
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Value");
         Ice.Object @object = new InitialI(adapter);
-        adapter.add(@object, Ice.Util.stringToIdentity("initial"));
+        adapter.add(@object, communicator().stringToIdentity("initial"));
         adapter.activate();
         communicator().waitForShutdown();
         return 0;
@@ -22,7 +22,10 @@ public class Server : Ice.Application
     public static void Main(string[] args)
     {
         Server app = new Server();
-        int status = app.main(args, "config");
-        System.Environment.Exit(status);
+        int status = app.main(args, "config.server");
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
