@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,7 +14,7 @@ class HelloServiceI : Ice.LocalObjectImpl, IceBox.Service
     public void start(string name, Ice.Communicator communicator, string[] args)
     {
         _adapter = communicator.createObjectAdapter(name);
-        _adapter.add(new HelloI(), Ice.Util.stringToIdentity("hello"));
+        _adapter.add(new HelloI(), communicator.stringToIdentity("hello"));
         _adapter.activate();
     }
 
