@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,7 +13,7 @@ public class Server : Ice.Application
     {
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("Hello");
 	Ice.Properties properties = communicator().getProperties();
-	Ice.Identity id = Ice.Util.stringToIdentity(properties.getProperty("Identity"));
+	Ice.Identity id = communicator().stringToIdentity(properties.getProperty("Identity"));
         adapter.add(new HelloI(properties.getProperty("Ice.ServerId")), id);
         adapter.activate();
         communicator().waitForShutdown();
@@ -24,6 +24,9 @@ public class Server : Ice.Application
     {
 	Server app = new Server();
 	int status = app.main(args);
-        System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
