@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -39,6 +39,9 @@ public class Client
 		status = 1;
 	    }
 	}
-        Environment.Exit(status);
+	if(status != 0)
+	{
+	    System.Environment.Exit(status);
+	}
     }
 }
