@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -31,7 +31,7 @@ public class Server
 	    Ice.Object obj = new PrinterI();
 	    adapter.add(
 		    obj,
-		    Ice.Util.stringToIdentity("SimplePrinter"));
+		    ic.stringToIdentity("SimplePrinter"));
 	    adapter.activate();
 	    ic.waitForShutdown();
         } catch (Exception e) {
@@ -49,6 +49,9 @@ public class Server
 		status = 1;
 	    }
 	}
-        Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
