@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -23,7 +23,7 @@ public class FileI : FileDisp_
 
 	// Create an identity
 	//
-	Ice.Identity myID = Ice.Util.stringToIdentity(Ice.Util.generateUUID());
+	Ice.Identity myID = _adapter.getCommunicator().stringToIdentity(Ice.Util.generateUUID());
 
 	// Add the identity to the object adapter
 	//
