@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -21,11 +21,11 @@ using System.Runtime.CompilerServices;
 [assembly: AssemblyCopyright("Copyright (c) 2003-2005, ZeroC, Inc.")]
 [assembly: AssemblyTrademark("Ice")]
 [assembly: AssemblyCulture("")]		
-[assembly: AssemblyVersion("3.0.1")]
+[assembly: AssemblyVersion("3.1.1")]
 [assembly: AssemblyDelaySign(false)]
 #if __MonoCS__
 [assembly: AssemblyKeyFile("../../config/IcecsKey.snk")] // mcs uses different search algorithm.
-#else
+#elif ICE_DOTNET_1X
 [assembly: AssemblyKeyFile("../../../../config/IcecsKey.snk")]
 #endif
 [assembly: AssemblyKeyName("")]
