@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -718,12 +718,12 @@ namespace IceInternal
 	    return this;
 	}
 
-	internal byte[] rawBytes()
+	public byte[] rawBytes()
 	{
 	    return _bytes;
 	}
 
-        internal byte[] rawBytes(int offset, int len)
+        public byte[] rawBytes(int offset, int len)
         {
             if(offset + len > _limit)
             {
