@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -80,10 +80,11 @@ namespace IceInternal
 	    }
 	}
 	
-	public Ice.ConnectionI create(EndpointI[] endpts, out bool compress)
+	public Ice.ConnectionI create(EndpointI[] endpts, bool hasMore, out bool compress)
 	{
 	    Debug.Assert(endpts.Length > 0);
 	    EndpointI[] endpoints = new EndpointI[endpts.Length];
+
 	    for(int i = 0; i < endpoints.Length; ++i)
 	    {
 		endpoints[i] = endpts[i];
@@ -239,7 +240,6 @@ namespace IceInternal
                                     {
                                         compress = endpts[i].compress();
                                     }
-
                                     return conn;
                                 }
                             }
@@ -323,7 +323,7 @@ namespace IceInternal
 		{
 		    System.Text.StringBuilder s = new System.Text.StringBuilder();
 		    s.Append("connection to endpoint failed");
-		    if(i < endpoints.Length - 1)
+		    if(hasMore || i < endpoints.Length - 1)
 		    {
 			s.Append(", trying next endpoint\n");
 		    }
@@ -332,7 +332,7 @@ namespace IceInternal
 			s.Append(" and no more endpoints to try\n");
 		    }
 		    s.Append(exception);
-		    instance_.logger().trace(traceLevels.retryCat, s.ToString());
+		    instance_.initializationData().logger.trace(traceLevels.retryCat, s.ToString());
 		}
 	    }
 	    
@@ -448,8 +448,8 @@ namespace IceInternal
 	    lock(this)
 	    {
 		if(_destroyed)
-		{
-		    throw new Ice.CommunicatorDestroyedException();
+		{ 
+		    return;
 		}
 		
 		foreach(LinkedList connectionList in _connections.Values)
@@ -855,14 +855,16 @@ namespace IceInternal
 	    return _acceptor.ToString();
 	}
 	
-	public IncomingConnectionFactory(Instance instance, EndpointI endpoint, Ice.ObjectAdapter adapter)
+	public IncomingConnectionFactory(Instance instance, EndpointI endpoint, Ice.ObjectAdapter adapter,
+					 string adapterName)
 	    : base(instance)
 	{
 	    _endpoint = endpoint;
 	    _adapter = adapter;
 	    _registeredWithPool = false;
 	    _finishedCount = 0;
-	    _warn = instance_.properties().getPropertyAsInt("Ice.Warn.Connections") > 0 ? true : false;
+	    _warn = 
+	        instance_.initializationData().properties.getPropertyAsInt("Ice.Warn.Connections") > 0 ? true : false;
 	    _connections = new LinkedList();
 	    _state = StateHolding;
 	    
@@ -913,7 +915,7 @@ namespace IceInternal
 		else
 		{
 		    h = _endpoint;
-		    _acceptor = _endpoint.acceptor(ref h);
+		    _acceptor = _endpoint.acceptor(ref h, adapterName);
 		    _endpoint = h;
 		    Debug.Assert(_acceptor != null);
 		    _acceptor.listen();
@@ -933,12 +935,41 @@ namespace IceInternal
 			}
 			catch(System.Exception ex)
 			{
-			    instance_.logger().error("cannot create thread for incoming connection factory:\n" + ex);
+			    instance_.initializationData().logger.error(
+			    	"cannot create thread for incoming connection factory:\n" + ex);
 			    throw;
 			}
 		    }
 		}
 	    }
+	    catch(Ice.LocalException)
+	    {
+		//
+		// Clean up for finalizer.
+		//
+		
+		if(_acceptor != null)
+		{
+		    try
+		    {
+			_acceptor.close();
+		    }
+		    catch(Ice.LocalException)
+		    {
+			// Here we ignore any exceptions in close().			
+		    }
+		}
+
+		lock(this)
+		{
+		    _state = StateClosed;
+		    _acceptor = null;
+		    _connections = null;
+		    _threadPerIncomingConnectionFactory = null;
+		}
+
+		throw;
+	    }
 	    catch(System.Exception ex)
 	    {
 		//
@@ -1095,7 +1126,7 @@ namespace IceInternal
 	
 	private void warning(Ice.LocalException ex)
 	{
-	    instance_.logger().warning("connection exception:\n" + ex + '\n' + _acceptor.ToString());
+	    instance_.initializationData().logger.warning("connection exception:\n" + ex + '\n' + _acceptor.ToString());
 	}
 
 	private void run()
@@ -1220,12 +1251,13 @@ namespace IceInternal
 	    }
 	    catch(Ice.Exception ex)
 	    {
-		instance_.logger().error("exception in thread per incoming connection factory:\n" + ToString() +
-					 ex.ToString());
+		instance_.initializationData().logger.error("exception in thread per incoming connection factory:\n" +
+					ToString() + ex.ToString());
 	    }
 	    catch(System.Exception ex)
 	    {
-		instance_.logger().error("system exception in thread per incoming connection factory:\n" + ToString() +
+		instance_.initializationData().logger.error(
+					"system exception in thread per incoming connection factory:\n" + ToString() +
 					 ex.ToString());
 	    }
 	}
