@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -204,6 +204,11 @@ namespace Ice
 	{
 	    lock(this)
 	    {
+		while(_state == StateNotValidated)
+		{
+		    Monitor.Wait(this);
+		}
+
 		setState(StateActive);
 	    }
 	}
@@ -212,6 +217,11 @@ namespace Ice
 	{
 	    lock(this)
 	    {
+		while(_state == StateNotValidated)
+		{
+		    Monitor.Wait(this);
+		}
+
 		setState(StateHolding);
 	    }
 	}
@@ -316,6 +326,18 @@ namespace Ice
 	    return true;
 	}
 
+	public void throwException()
+	{
+	    lock(this)
+	    {
+	        if(_exception != null)
+		{
+		    Debug.Assert(_state >= StateClosing);
+		    throw _exception;
+		}
+	    }
+	}
+
 	public void waitUntilHolding()
 	{
 	    lock(this)
@@ -448,26 +470,6 @@ namespace Ice
 	    }
 	}
 	
-	private static readonly byte[] _requestHdr = new byte[]
-	{
-	    IceInternal.Protocol.magic[0], IceInternal.Protocol.magic[1], IceInternal.Protocol.magic[2], 
-	    IceInternal.Protocol.magic[3],
-	    IceInternal.Protocol.protocolMajor, IceInternal.Protocol.protocolMinor,
-	    IceInternal.Protocol.encodingMajor, IceInternal.Protocol.encodingMinor,
-	    IceInternal.Protocol.requestMsg,
-	    (byte)0, // Compression status.
-	    (byte)0, (byte)0, (byte)0, (byte)0, // Message size (placeholder).
-	    (byte)0, (byte)0, (byte)0, (byte)0	// Request ID (placeholder).
-	};
-	
-	//
-	// TODO: Should not be a member function of ConnectionI.
-	//
-	public void prepareRequest(IceInternal.BasicStream os)
-	{
-	    os.writeBlob(_requestHdr);
-	}
-
 	private IceInternal.BasicStream doCompress(IceInternal.BasicStream uncompressed, bool compress)
 	{
 	    if(_compressionSupported)
@@ -547,7 +549,12 @@ namespace Ice
 
 		if(_exception != null)
 		{
-		    throw _exception;
+		    //
+		    // If the connection is closed before we even have a chance
+		    // to send our request, we always try to send the request
+		    // again.
+		    //
+		    throw new IceInternal.LocalExceptionWrapper(_exception, true);
 		}
 
 		Debug.Assert(_state > StateNotValidated);
@@ -657,7 +664,12 @@ namespace Ice
 
 		if(_exception != null)
 		{
-		    throw _exception;
+		    //
+		    // If the connection is closed before we even have a chance
+		    // to send our request, we always try to send the request
+		    // again.
+		    //
+		    throw new IceInternal.LocalExceptionWrapper(_exception, true);
 		}
 
 		Debug.Assert(_state > StateNotValidated);
@@ -742,18 +754,6 @@ namespace Ice
 	    }
 	}
 	
-	private static readonly byte[] _requestBatchHdr = new byte[]
-	{
-	    IceInternal.Protocol.magic[0], IceInternal.Protocol.magic[1], IceInternal.Protocol.magic[2],
-	    IceInternal.Protocol.magic[3],
-	    IceInternal.Protocol.protocolMajor, IceInternal.Protocol.protocolMinor,
-	    IceInternal.Protocol.encodingMajor, IceInternal.Protocol.encodingMinor,
-	    IceInternal.Protocol.requestBatchMsg,
-	    (byte)0, // Compression status.
-	    (byte)0, (byte)0, (byte)0, (byte)0, // Message size (placeholder).
-	    (byte)0, (byte)0, (byte)0, (byte)0	// Number of requests in batch (placeholder).
-	};
-	
 	public void prepareBatchRequest(IceInternal.BasicStream os)
 	{
 	    lock(this)
@@ -778,7 +778,7 @@ namespace Ice
 		{
 		    try
 		    {
-			_batchStream.writeBlob(_requestBatchHdr);
+			_batchStream.writeBlob(IceInternal.Protocol.requestBatchHdr);
 		    }
 		    catch(LocalException ex)
 		    {
@@ -1050,6 +1050,10 @@ namespace Ice
 		if(_adapter != null)
 		{
 		    _servantManager = ((ObjectAdapterI) _adapter).getServantManager();
+		    if(_servantManager == null)
+		    {
+		        _adapter = null;
+		    }
 		}
 		else
 		{
@@ -1081,7 +1085,8 @@ namespace Ice
 	    ConnectionI[] connections = new ConnectionI[1];
 	    connections[0] = this;
 	    IceInternal.Reference @ref = instance_.referenceFactory().create(
-		ident, instance_.getDefaultContext(), "", IceInternal.Reference.Mode.ModeTwoway, connections);
+		ident, instance_.getDefaultContext(), "", IceInternal.Reference.Mode.ModeTwoway,
+		connections);
 	    return instance_.proxyFactory().referenceToProxy(@ref);
 	}
 	
@@ -1115,17 +1120,6 @@ namespace Ice
 	    //
 	}
 	
-	private static readonly byte[] _replyHdr = new byte[]
-	{
-	    IceInternal.Protocol.magic[0], IceInternal.Protocol.magic[1], IceInternal.Protocol.magic[2], 
-	    IceInternal.Protocol.magic[3],
-	    IceInternal.Protocol.protocolMajor, IceInternal.Protocol.protocolMinor,
-	    IceInternal.Protocol.encodingMajor, IceInternal.Protocol.encodingMinor,
-	    IceInternal.Protocol.replyMsg,
-	    (byte)0, // Compression status.
-	    (byte)0, (byte)0, (byte)0, (byte)0 // Message size (placeholder).
-	};
-	
 	public override void message(IceInternal.BasicStream stream, IceInternal.ThreadPool threadPool)
 	{
 	    Debug.Assert(!instance_.threadPerConnection()); // Only for use with a thread pool.
@@ -1274,7 +1268,19 @@ namespace Ice
 	{
 	    return _desc; // No mutex lock, _desc is immutable.
 	}
-	
+
+	//
+	// Only used by the SSL plug-in.
+	//
+	// The external party has to synchronize the connection, since the
+	// connection is the object that protects the transceiver.
+	//
+	public IceInternal.Transceiver
+	getTransceiver()
+	{
+	    return _transceiver;
+	}
+
 	static ConnectionI()
 	{
 	    _compressionSupported = IceInternal.BasicStream.compressible();
@@ -1289,11 +1295,11 @@ namespace Ice
 	    _type = transceiver.type();
 	    _endpoint = endpoint;
 	    _adapter = adapter;
-	    _logger = instance.logger(); // Cached for better performance.
+	    _logger = instance.initializationData().logger; // Cached for better performance.
 	    _traceLevels = instance.traceLevels(); // Cached for better performance.
 	    _registeredWithPool = false;
 	    _finishedCount = 0;
-	    _warn = instance_.properties().getPropertyAsInt("Ice.Warn.Connections") > 0;
+	    _warn = instance_.initializationData().properties.getPropertyAsInt("Ice.Warn.Connections") > 0;
 	    _acmAbsoluteTimeoutMillis = 0;
 	    _nextRequestId = 1;
 	    _batchStream = new IceInternal.BasicStream(instance);
@@ -1320,7 +1326,8 @@ namespace Ice
 		}
 	    }
 
-	    _compressionLevel = instance_.properties().getPropertyAsIntWithDefault("Ice.Compression.Level", 1);
+	    _compressionLevel = 
+	        instance_.initializationData().properties.getPropertyAsIntWithDefault("Ice.Compression.Level", 1);
 	    if(_compressionLevel < 1)
 	    {
 		_compressionLevel = 1;
@@ -1369,11 +1376,11 @@ namespace Ice
 	    {
 		if(instance_.threadPerConnection())
 		{
-		    instance_.logger().error("cannot create thread for connection:\n" + ex);
+		    _logger.error("cannot create thread for connection:\n" + ex);
 		}
 		else
 		{
-		    instance_.logger().error("cannot create thread pool for connection:\n" + ex);
+		    _logger.error("cannot create thread pool for connection:\n" + ex);
 		}
 
 		try
@@ -1896,7 +1903,7 @@ namespace Ice
 		    // Prepare the invocation.
 		    //
 		    bool response = !_endpoint.datagram() && requestId != 0;
-		    inc = getIncoming(adapter, response, compress);
+		    inc = getIncoming(adapter, response, compress, requestId);
 		    IceInternal.BasicStream ins = inc.istr();
 		    stream.swap(ins);
 		    IceInternal.BasicStream os = inc.ostr();
@@ -1907,7 +1914,7 @@ namespace Ice
 		    if(response)
 		    {
 			Debug.Assert(invokeNum == 1); // No further invocations if a response is expected.
-			os.writeBlob(_replyHdr);
+			os.writeBlob(IceInternal.Protocol.replyHdr);
 			
 			//
 			// Add the request ID.
@@ -1999,16 +2006,19 @@ namespace Ice
 			//
 			lock(_sendMutex)
 			{
-			    try
+			    if(_transceiver != null)
 			    {
-				_transceiver.close();
-			    }
-			    catch(LocalException)
-			    {
-				// Here we ignore any exceptions in close().
-			    }
+				try
+				{
+				    _transceiver.close();
+				}
+				catch(LocalException)
+				{
+				    // Here we ignore any exceptions in close().
+				}
 
-			    _transceiver = null;
+				_transceiver = null;
+			    }
 			    Monitor.PulseAll(this);
 			    return;
 			}
@@ -2018,7 +2028,7 @@ namespace Ice
 		activate();
 	    }
 
-	    bool warnUdp = instance_.properties().getPropertyAsInt("Ice.Warn.Datagrams") > 0;
+	    bool warnUdp = instance_.initializationData().properties.getPropertyAsInt("Ice.Warn.Datagrams") > 0;
 
 	    bool closed = false;
 
@@ -2040,7 +2050,13 @@ namespace Ice
 			_transceiver.read(stream, -1);
 
 			int pos = stream.pos();
-			Debug.Assert(pos >= IceInternal.Protocol.headerSize);
+			if(pos < IceInternal.Protocol.headerSize)
+			{
+			    //
+			    // This situation is possible for small UDP packets.
+			    //
+			    throw new IllegalMessageSizeException();
+			}
 			stream.pos(0);
 			byte[] m = stream.readBlob(4);
 			if(m[0] != IceInternal.Protocol.magic[0] || m[1] != IceInternal.Protocol.magic[1] ||
@@ -2245,21 +2261,34 @@ namespace Ice
 
 	public void ThreadPerConnection()
 	{
+	    if(instance_.initializationData().threadHook != null)
+	    {
+	        instance_.initializationData().threadHook.start();
+	    }
+
 	    try
 	    {
 		run();
 	    }
 	    catch(Exception ex)
 	    {
-		instance_.logger().error("exception in thread per connection:\n" + ToString() + ex.ToString());
+		_logger.error("exception in thread per connection:\n" + ToString() + "\n" + ex.ToString());
 	    }
 	    catch(System.Exception ex)
 	    {
-		instance_.logger().error("system exception in thread per connection:\n" + ToString() + ex.ToString());
+		_logger.error("system exception in thread per connection:\n" + ToString() + "\n" +
+					 ex.ToString());
+	    }
+	    finally
+	    {
+	        if(instance_.initializationData().threadHook != null)
+	        {
+	            instance_.initializationData().threadHook.stop();
+	        }
 	    }
 	}
 
-	private IceInternal.Incoming getIncoming(ObjectAdapter adapter, bool response, byte compress)
+	private IceInternal.Incoming getIncoming(ObjectAdapter adapter, bool response, byte compress, int requestId)
 	{
 	    IceInternal.Incoming inc = null;
 	    
@@ -2267,13 +2296,13 @@ namespace Ice
 	    {
 		if(_incomingCache == null)
 		{
-		    inc = new IceInternal.Incoming(instance_, this, adapter, response, compress);
+		    inc = new IceInternal.Incoming(instance_, this, adapter, response, compress, requestId);
 		}
 		else
 		{
 		    inc = _incomingCache;
 		    _incomingCache = _incomingCache.next;
-		    inc.reset(instance_, this, adapter, response, compress);
+		    inc.reset(instance_, this, adapter, response, compress, requestId);
 		    inc.next = null;
 		}
 	    }
