@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,7 +14,7 @@ namespace IceInternal
     {
 	short type();
 	string protocol();
-	EndpointI create(string str, bool adapterEndp);
+	EndpointI create(string str);
 	EndpointI read(BasicStream s);
 	void destroy();
     }
