@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,6 +17,7 @@ namespace IceInternal
     using System.Net;
     using System.Net.Sockets;
     using System.Runtime.InteropServices;
+    using System.Threading;
 
     public sealed class Network
     {
@@ -101,6 +102,15 @@ namespace IceInternal
 		   error == WSAENETRESET;
 	}
 
+	public static bool connectionLost(System.IO.IOException ex)
+	{
+	    string msg = ex.Message.ToLower();
+	    return msg.IndexOf("connection was forcibly closed") >= 0 ||
+		   msg.IndexOf("remote party has closed the transport stream") >= 0 ||
+		   msg.IndexOf("established connection was aborted") >= 0 ||
+		   msg.IndexOf("received an unexpected eof or 0 bytes from the transport stream") >= 0;
+	}
+
 	public static bool connectionRefused(Win32Exception ex)
 	{
 	    return ex.NativeErrorCode == WSAECONNREFUSED;
@@ -117,6 +127,11 @@ namespace IceInternal
 	    return ex.NativeErrorCode == WSAEMSGSIZE;
 	}
 
+	public static bool timeout(System.IO.IOException ex)
+	{
+	    return ex.Message.IndexOf("period of time") >= 0;
+	}
+
 	public static Socket createSocket(bool udp)
 	{
 	    Socket socket;
@@ -153,7 +168,7 @@ namespace IceInternal
 	    return socket;
 	}
 
-	private static void closeSocketNoThrow(Socket socket)
+	public static void closeSocketNoThrow(Socket socket)
 	{
 	    if(socket == null)
 	    {
@@ -385,33 +400,23 @@ namespace IceInternal
 		    {
 			closeSocketNoThrow(socket);
 
-			//
-			// BUGFIX: Mono 1.9.1 appears to be returning native error numbers, not the WSA
-			// variety. http://bugzilla.ximian.com/show_bug.cgi?id=76496
-			//
-			// Unfortunately the native code for ECONNREFUSED under MacOS is 61. Under Linux
-			// its 111 -- they don't match. Therefore we assume ConnectFailedException.
-			//
-			throw new Ice.ConnectFailedException("Connect failed");
-
 			//
 			// Create a Win32Exception out of this error value. Check the for refused, and
 			// failed. Otherwise its a plain old socket exception.
 			//
-			//Win32Exception sockEx = new Win32Exception(val);
-			//if(connectionRefused(sockEx))
-			//{
-			    //throw new Ice.ConnectionRefusedException("Connect refused", sockEx);
-			//}
-			//else if(connectFailed(sockEx))
-			//{
-			    //throw new Ice.ConnectFailedException("Connect failed", sockEx);
-			//}
-			//else
-			//{
-			    //throw new Ice.SocketException(sockEx);
-			//}
-			
+			Win32Exception sockEx = new Win32Exception(val);
+			if(connectionRefused(sockEx))
+			{
+			    throw new Ice.ConnectionRefusedException("Connect refused", sockEx);
+			}
+			else if(connectFailed(sockEx))
+			{
+			    throw new Ice.ConnectFailedException("Connect failed", sockEx);
+			}
+			else
+			{
+			    throw new Ice.SocketException(sockEx);
+			}			
 		    }
 		    
 		    //
@@ -462,7 +467,101 @@ namespace IceInternal
 		}
 	    }
 	}
-	
+
+	internal class AsyncConnectInfo
+	{
+	    internal AsyncConnectInfo(Socket fd)
+	    {
+		this.fd = fd;
+		this.ex = null;
+		this.done = false;
+	    }
+
+	    internal Socket fd;
+	    volatile internal Exception ex;
+	    volatile internal bool done;
+	}
+
+	private static void asyncConnectCallback(IAsyncResult ar)
+	{
+	    AsyncConnectInfo info = (AsyncConnectInfo)ar.AsyncState;
+	    lock(info)
+	    {
+		try
+		{
+		    info.fd.EndConnect(ar);
+		}
+		catch(Exception ex)
+		{
+		    info.ex = ex;
+		}
+		finally
+		{
+		    info.done = true;
+		    Monitor.Pulse(info);
+		}
+	    }
+	}
+
+	public static void doConnectAsync(Socket socket, EndPoint addr, int timeout)
+	{
+            //
+            // Set larger send buffer size to avoid performance problems on
+            // WIN32.
+            //
+	    if(AssemblyUtil.platform_ == AssemblyUtil.Platform.Windows)
+	    {
+		setSendBufferSize(socket, 64 * 1024);
+	    }
+
+	repeatConnect:
+	    try
+	    {
+		AsyncConnectInfo info = new AsyncConnectInfo(socket);
+		/* IAsyncResult ar = */ socket.BeginConnect(addr, new AsyncCallback(asyncConnectCallback), info);
+		lock(info)
+		{
+		    if(!info.done)
+		    {
+			if(!Monitor.Wait(info, timeout == -1 ? Timeout.Infinite : timeout))
+			{
+			    throw new Ice.ConnectTimeoutException("Connect timed out after " + timeout + " msec");
+			}
+		    }
+		    if(info.ex != null)
+		    {
+			throw info.ex;
+		    }
+		}
+	    }
+	    catch(SocketException ex)
+	    {
+		if(interrupted(ex))
+		{
+		    goto repeatConnect;
+		}
+
+		closeSocketNoThrow(socket);
+
+		//
+		// Check for connectionRefused, and connectFailed
+		// here.
+		//
+		if(connectionRefused(ex))
+		{
+		    throw new Ice.ConnectionRefusedException("Connect refused", ex);
+		}
+		else if(connectFailed(ex))
+		{
+		    throw new Ice.ConnectFailedException("Connection failed", ex);
+		}
+		else
+		{
+		    throw;
+		}
+	    }
+	}
+
 	public static Socket doAccept(Socket socket, int timeout)
 	{
 	    Socket ret = null;
@@ -636,7 +735,18 @@ namespace IceInternal
 	repeatGetHostByName:
 	    try
 	    {
+#if ICE_DOTNET_1X
 		IPHostEntry e = Dns.GetHostByName(host);
+#else
+                try
+                {
+                    return new IPEndPoint(IPAddress.Parse(host), port);
+                }
+                catch (FormatException)
+                {
+                }
+		IPHostEntry e = Dns.GetHostEntry(host);
+#endif
 		Debug.Assert(e.AddressList.Length != 0);
 		return new IPEndPoint(e.AddressList[0], port);
 	    }
@@ -646,52 +756,18 @@ namespace IceInternal
 		{
 		    goto repeatGetHostByName;
 		}
-		Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+		Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 		e.host = host;
 		throw e;
 	    }
 	    catch(System.Exception ex)
 	    {
-		Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+		Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 		e.host = host;
 		throw e;
 	    }
 	}
 	
-	public static string getLocalHost(bool numeric)
-	{
-	    string hostname;
-
-	    int retry = 5;
-
-	repeatGetHostName:
-	    try
-	    {
-		hostname = Dns.GetHostName();
-	    }
-	    catch(Win32Exception ex)
-	    {
-		if(ex.NativeErrorCode == WSATRY_AGAIN && --retry >= 0)
-		{
-		    goto repeatGetHostName;
-		}
-		Ice.DNSException e = new Ice.DNSException("GetHostName failed", ex);
-		throw e;
-	    }
-	    catch(System.Exception ex)
-	    {
-		Ice.DNSException e = new Ice.DNSException("GetHostName failed", ex);
-		throw e;
-	    }
-	    
-	    if(numeric)
-	    {
-	        hostname = getNumericHost(hostname);
-	    }
-
-	    return hostname;
-	}
-
 	public static string getNumericHost(string hostname)
 	{
 	    int retry = 5;
@@ -700,7 +776,11 @@ namespace IceInternal
 	    string numericHost;
 	    try
 	    {
+#if ICE_DOTNET_1X
 	        numericHost = Dns.GetHostByName(hostname).AddressList[0].ToString();
+#else
+	        numericHost = Dns.GetHostEntry(hostname).AddressList[0].ToString();
+#endif
 	    }
 	    catch(Win32Exception ex)
 	    {
@@ -708,13 +788,13 @@ namespace IceInternal
 	        {
 	    	    goto repeatGetHostByName;
 	        }
-	        Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+	        Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 	        e.host = hostname;
 	        throw e;
 	    }
 		catch(System.Exception ex)
 	    {
-	        Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+	        Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 	        e.host = hostname;
 	        throw e;
 	    }
@@ -730,7 +810,11 @@ namespace IceInternal
 	repeatGetHostByName:
 	    try
 	    {
+#if ICE_DOTNET_1X
 	        IPHostEntry e = Dns.GetHostByName(Dns.GetHostName());
+#else
+	        IPHostEntry e = Dns.GetHostEntry(Dns.GetHostName());
+#endif
 		hosts = new string[e.AddressList.Length + 1];
 		for(int i = 0; i < e.AddressList.Length; ++i)
 		{
@@ -744,13 +828,13 @@ namespace IceInternal
 	        {
 	    	goto repeatGetHostByName;
 	        }
-	        Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+	        Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 	        e.host = "0.0.0.0";
 	        throw e;
 	    }
 	    catch(System.Exception ex)
 	    {
-	        Ice.DNSException e = new Ice.DNSException("GetHostByName failed", ex);
+	        Ice.DNSException e = new Ice.DNSException("address lookup failed", ex);
 	        e.host = "0.0.0.0";
 	        throw e;
 	    }
@@ -768,7 +852,6 @@ namespace IceInternal
 		sink = createSocket(false);
 		Socket listener = createSocket(false);
 
-
 		doBind(listener, new IPEndPoint(IPAddress.Parse("127.0.0.1"), 0));
 		doListen(listener, 1);
 		doConnect(sink, listener.LocalEndPoint, 1000);
