@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -37,134 +37,84 @@ namespace Ice
 	{
 	    IceInternal.LocatorInfo locatorInfo = null;
 	    bool registerProcess = false;
-	    string serverId = "";
-	    Communicator communicator = null;
 	    bool printAdapterReady = false;
 
 	    lock(this)
 	    {
 		checkForDeactivation();
 		
-		if(!_printAdapterReadyDone)
+		//
+		// If the one off initializations of the adapter are already
+		// done, we just need to activate the incoming connection
+		// factories and we're done.
+		//
+		if(_activateOneOffDone)
 		{
-		    locatorInfo = _locatorInfo;
-		    registerProcess = instance_.properties().getPropertyAsInt(_name + ".RegisterProcess") > 0;
-		    serverId = instance_.properties().getProperty("Ice.ServerId");
-		    printAdapterReady = instance_.properties().getPropertyAsInt("Ice.PrintAdapterReady") > 0;
-		    communicator = _communicator;
-		    _printAdapterReadyDone = true;
+		    foreach(IceInternal.IncomingConnectionFactory icf in _incomingConnectionFactories)
+		    {
+			icf.activate();
+		    }
+		    return;
 		}
+
+		//
+		// One off initializations of the adapter: update the locator
+		// registry and print the "adapter ready" message. We set the
+		// _waitForActivate flag to prevent deactivation from other
+		// threads while these one off initializations are done.
+		//
+		_waitForActivate = true;
 		
+		locatorInfo = _locatorInfo;
+		Properties properties = instance_.initializationData().properties;
+		registerProcess = properties.getPropertyAsInt(_name + ".RegisterProcess") > 0;
+		printAdapterReady = properties.getPropertyAsInt("Ice.PrintAdapterReady") > 0;
+	    }
+	    
+	    try
+	    {
+		Ice.Identity dummy = new Ice.Identity();
+		dummy.name = "dummy";
+		updateLocatorRegistry(locatorInfo, createDirectProxy(dummy), registerProcess);
+	    }
+	    catch(Ice.LocalException ex)
+	    {
+		//
+		// If we couldn't update the locator registry, we let the
+		// exception go through and don't activate the adapter to
+		// allow to user code to retry activating the adapter
+		// later.
+		//
+		lock(this)
+		{
+		    _waitForActivate = false;
+		    System.Threading.Monitor.PulseAll(this);
+		}
+		throw ex;
+	    }
+		
+	    if(printAdapterReady)
+	    {
+		System.Console.Out.WriteLine(_name + " ready");
+	    }
+
+	    lock(this)
+	    {
+		Debug.Assert(!_deactivated); // Not possible if _waitForActivate = true;
+	    
+		//
+		// Signal threads waiting for the activation.
+		//
+		_waitForActivate = false;
+		System.Threading.Monitor.PulseAll(this);
+
+		_activateOneOffDone = true;
+	    
 		foreach(IceInternal.IncomingConnectionFactory icf in _incomingConnectionFactories)
 		{
 		    icf.activate();
 		}
 	    }
-
-	    if(registerProcess || _id.Length > 0)
-	    {
-		//
-		// We must get and call on the locator registry outside the thread
-		// synchronization to avoid deadlocks. (we can't make remote calls
-		// within the OA synchronization because the remote call will
-		// indirectly call isLocal() on this OA with the OA factory
-		// locked).
-		//
-
-		LocatorRegistryPrx locatorRegistry = null;
-		if(locatorInfo != null)
-		{
-		    //
-		    // TODO: This might throw if we can't connect to the
-		    // locator. Shall we raise a special exception for the
-		    // activate operation instead of a non obvious network
-		    // exception?
-		    //
-		    locatorRegistry = locatorInfo.getLocatorRegistry();
-		}
-	    
-		if(locatorRegistry != null && _id.Length > 0)
-		{
-		    try 
-		    {
-			Identity ident = new Identity();
-			ident.category = "";
-			ident.name = "dummy";
-			if(_replicaGroupId.Length == 0)
-			{
-			    locatorRegistry.setAdapterDirectProxy(_id, createDirectProxy(ident));
-			}
-			else
-			{
-			    locatorRegistry.setReplicatedAdapterDirectProxy(_id, _replicaGroupId, 
-									    createDirectProxy(ident));
-			}
-		    }
-		    catch(Ice.ObjectAdapterDeactivatedException)
-		    {
-			// IGNORE: The object adapter is already inactive.
-		    }
-		    catch(Ice.AdapterNotFoundException)
-		    {
-			NotRegisteredException ex1 = new NotRegisteredException();
-			ex1.kindOfObject = "object adapter";
-			ex1.id = _id;
-			throw ex1;
-		    }
-		    catch(Ice.InvalidReplicaGroupIdException)
-		    {
-			NotRegisteredException ex1 = new NotRegisteredException();
-			ex1.kindOfObject = "replica group";
-			ex1.id = _replicaGroupId;
-			throw ex1;
-		    }
-		    catch(Ice.AdapterAlreadyActiveException)
-		    {
-			ObjectAdapterIdInUseException ex1 = new ObjectAdapterIdInUseException();
-			ex1.id = _id;
-			throw ex1;
-		    }
-		}
-
-		if(registerProcess)
-		{
-		    if(locatorRegistry == null)
-		    {
-			communicator.getLogger().warning("object adapter `" + _name + "' cannot register the " +
-							 "process without alocator registry");
-		    }
-		    else if(serverId.Length == 0)
-		    {
-			communicator.getLogger().warning("object adapter `" + _name + "' cannot register the " +
-							 "process without a value for Ice.ServerId");
-		    }
-		    else
-		    {
-			try
-			{
-			    Process servant = new ProcessI(communicator);
-			    Ice.ObjectPrx proxy = createDirectProxy(addWithUUID(servant).ice_getIdentity());
-			    locatorRegistry.setServerProcessProxy(serverId, ProcessPrxHelper.uncheckedCast(proxy));
-			} 
-			catch(Ice.ObjectAdapterDeactivatedException)
-			{
-			    // IGNORE: The object adapter is already inactive.
-			}
-			catch(ServerNotFoundException)
-			{
-			    NotRegisteredException ex1 = new NotRegisteredException();
-			    ex1.id = serverId;
-			    ex1.kindOfObject = "server";
-			    throw ex1;
-			}
-		    }
-		}
-	    }
-
-	    if(printAdapterReady)
-	    {
-		System.Console.Out.WriteLine(_name + " ready");
-	    }
 	}
 	
 	public void hold()
@@ -203,6 +153,7 @@ namespace Ice
 	{
 	    IceInternal.OutgoingConnectionFactory outgoingConnectionFactory;
 	    ArrayList incomingConnectionFactories;
+	    IceInternal.LocatorInfo locatorInfo;
 
 	    lock(this)
 	    {
@@ -214,15 +165,51 @@ namespace Ice
 		{
 		    return;
 		}
+
+		//
+		//
+		// Wait for activation to complete. This is necessary to not 
+		// get out of order locator updates.
+		//
+		while(_waitForActivate)
+		{
+		    System.Threading.Monitor.Wait(this);
+		}
+
+		if(_routerInfo != null)
+		{
+		    //
+		    // Remove entry from the router manager.
+		    //
+		    instance_.routerManager().erase(_routerInfo.getRouter());
+
+		    //
+		    // Clear this object adapter with the router.
+		    //
+		    _routerInfo.setAdapter(null);
+		}
 		
 		incomingConnectionFactories = new ArrayList(_incomingConnectionFactories);
 		outgoingConnectionFactory = instance_.outgoingConnectionFactory();
-		
+		locatorInfo = _locatorInfo;
+
 		_deactivated = true;
 		
 		System.Threading.Monitor.PulseAll(this);
 	    }
 
+	    try
+	    {
+		updateLocatorRegistry(locatorInfo, null, false);
+	    }
+	    catch(Ice.LocalException)
+	    {
+		//
+		// We can't throw exceptions in deactivate so we ignore
+		// failures to update the locator registry.
+		//
+	    }
+
 	    //
 	    // Must be called outside the thread synchronization, because
 	    // Connection::destroy() might block when sending a CloseConnection
@@ -289,7 +276,7 @@ namespace Ice
 	    // Now it's also time to clean up our servants and servant
 	    // locators.
 	    //
-	    if(_servantManager != null)
+	    if(instance_ != null) // Don't destroy twice.
 	    {
 		_servantManager.destroy();
 	    }
@@ -302,6 +289,8 @@ namespace Ice
 		_threadPool.destroy();
 		_threadPool.joinWithAllThreads();
 	    }
+
+	    IceInternal.ObjectAdapterFactory objectAdapterFactory;
 	    
 	    lock(this)
 	    {
@@ -325,16 +314,23 @@ namespace Ice
 		//
 		instance_ = null;
 		_threadPool = null;
-		_servantManager = null;
 		_communicator = null;
 		_incomingConnectionFactories = null;
 		_routerEndpoints = null;
-		_routerInfos = null;
+		_routerInfo = null;
 		_publishedEndpoints = null;
 		_locatorInfo = null;
+
+		objectAdapterFactory = _objectAdapterFactory;
+		_objectAdapterFactory = null;
+	    }
+
+	    if(objectAdapterFactory != null)
+	    {
+	        objectAdapterFactory.removeObjectAdapter(_name);
 	    }
 	}
-	
+
 	public ObjectPrx add(Ice.Object obj, Identity ident)
 	{
 	    return addFacet(obj, ident, "");
@@ -526,130 +522,12 @@ namespace Ice
 		    connections.CopyTo(arr, 0);
 		}
                 IceInternal.Reference @ref = instance_.referenceFactory().create(
-		    ident, new Ice.Context(), "", IceInternal.Reference.Mode.ModeTwoway, arr);
+		    ident, instance_.getDefaultContext(), "", IceInternal.Reference.Mode.ModeTwoway, 
+		    arr);
 		return instance_.proxyFactory().referenceToProxy(@ref);
 	    }
 	}
 	
-	public void addRouter(RouterPrx router)
-	{
-	    lock(this)
-	    {
-		checkForDeactivation();
-		
-		IceInternal.RouterInfo routerInfo = instance_.routerManager().get(router);
-		if(routerInfo != null)
-		{
-		    _routerInfos.Add(routerInfo);
-
-		    //
-		    // Add the router's server proxy endpoints to this object
-		    // adapter.
-		    //
-		    ObjectPrxHelperBase proxy = (ObjectPrxHelperBase)routerInfo.getServerProxy();
-		    IceInternal.EndpointI[] endpoints = proxy.reference__().getEndpoints();
-		    for(int i = 0; i < endpoints.Length; ++i)
-		    {
-			_routerEndpoints.Add(endpoints[i]);
-		    }
-		    _routerEndpoints.Sort(); // Must be sorted.
-
-		    //
-		    // Remove duplicate endpoints, so we have a list of unique endpoints.
-		    //
-		    for(int i = 0; i < _routerEndpoints.Count-1;)
-		    {
-			System.Object o1 = _routerEndpoints[i];
-			System.Object o2 = _routerEndpoints[i + 1];
-			if(o1.Equals(o2))
-			{
-			    _routerEndpoints.Remove(i);
-			}
-			else
-			{
-				++i;
-			}
-		    }
-
-		    //
-		    // Associate this object adapter with the router. This way,
-		    // new outgoing connections to the router's client proxy will
-		    // use this object adapter for callbacks.
-		    //
-		    routerInfo.setAdapter(this);
-		    
-		    //
-		    // Also modify all existing outgoing connections to the
-		    // router's client proxy to use this object adapter for
-		    // callbacks.
-		    //      
-		    instance_.outgoingConnectionFactory().setRouterInfo(routerInfo);
-		}
-	    }
-	}
-
-	public void removeRouter(RouterPrx router)
-	{
-	    lock(this)
-	    {
-		checkForDeactivation();
-
-		IceInternal.RouterInfo routerInfo = instance_.routerManager().erase(router);
-		if(routerInfo != null)
-		{
-		    //
-		    // Rebuild the router endpoints from our set of router infos.
-		    //
-		    _routerEndpoints.Clear();
-		    for(int p = 0; p < _routerInfos.Count;)
-		    {
-			if(_routerInfos[p] == routerInfo)
-			{
-			    _routerInfos.Remove(p);
-			    continue;
-			}
-			ObjectPrxHelperBase proxy = (ObjectPrxHelperBase)routerInfo.getServerProxy();
-			IceInternal.EndpointI[] endpoints = proxy.reference__().getEndpoints();
-			for(int i = 0; i < endpoints.Length; ++i)
-			{
-			    _routerEndpoints.Add(endpoints[i]);
-			}
-			++p;
-		    }
-
-		    _routerEndpoints.Sort(); // Must be sorted.
-		    //
-		    // Remove duplicate endpoints, so we have a list of unique endpoints.
-		    //
-		    for(int i = 0; i < _routerEndpoints.Count-1;)
-		    {
-			System.Object o1 = _routerEndpoints[i];
-			System.Object o2 = _routerEndpoints[i + 1];
-			if(o1.Equals(o2))
-			{
-			    _routerEndpoints.Remove(i);
-			}
-			else
-			{
-				++i;
-			}
-		    }
-
-		    //
-		    // Clear this object adapter with the router.
-		    //
-		    routerInfo.setAdapter(null);
-
-		    //
-		    // Also modify all existing outgoing connections to the
-		    // router's client proxy to use this object adapter for
-		    // callbacks.
-		    //	
-		    instance_.outgoingConnectionFactory().setRouterInfo(routerInfo);
-		}
-	    }
-	}
-	
 	public void setLocator(LocatorPrx locator)
 	{
 	    lock(this)
@@ -660,23 +538,6 @@ namespace Ice
 	    }
 	}
 
-	public LocatorPrx getLocator()
-	{
-	    lock(this)
-	    {
-		checkForDeactivation();
-
-		LocatorPrx locator = null;
-
-		if(_locatorInfo != null)
-		{
-		    locator = _locatorInfo.getLocator();
-		}
-
-		return locator;
-	    }
-	}
-
 	public bool isLocal(ObjectPrx proxy)
 	{
 	    IceInternal.Reference r = ((ObjectPrxHelperBase)proxy).reference__();
@@ -697,7 +558,7 @@ namespace Ice
 		if(info != null)
 		{
 		    bool isCached;
-		    endpoints = info.getEndpoints(ir, out isCached);
+		    endpoints = info.getEndpoints(ir, ir.getLocatorCacheTimeout(), out isCached);
 		}
 		else
 		{
@@ -737,11 +598,14 @@ namespace Ice
 		// router's server proxy endpoints (if any), are also considered
 		// local.
 		//
-		for(int i = 0; i < endpoints.Length; ++i)
+		if(_routerInfo != null && _routerInfo.getRouter().Equals(proxy.ice_getRouter()))
 		{
-		    if(_routerEndpoints.BinarySearch(endpoints[i]) >= 0) // _routerEndpoints is sorted.
+		    for(int i = 0; i < endpoints.Length; ++i)
 		    {
-			return true;
+		        if(_routerEndpoints.BinarySearch(endpoints[i]) >= 0) // _routerEndpoints is sorted.
+		        {
+			    return true;
+		        }
 		    }
 		}
 		
@@ -813,85 +677,159 @@ namespace Ice
 
 	public IceInternal.ServantManager getServantManager()
 	{
-	    // No mutex lock necessary, _threadPool and instance_ are
-	    // immutable after creation until they are removed in
-	    // waitForDeactivate().
-	    
-	    // Not check for deactivation here!
-	    
-	    Debug.Assert(instance_ != null); // Must not be called after waitForDeactivate().
-	    
+	    //
+	    // No mutex lock necessary, _servantManager is immutable.
+	    //
 	    return _servantManager;
 	}
 	
 	//
 	// Only for use by IceInternal.ObjectAdapterFactory
 	//
-	public ObjectAdapterI(IceInternal.Instance instance, Communicator communicator, string name, 
-			      string endpointInfo)
+	public ObjectAdapterI(IceInternal.Instance instance, Communicator communicator,
+			      IceInternal.ObjectAdapterFactory objectAdapterFactory, string name, 
+			      string endpointInfo, RouterPrx router)
 	{
 	    _deactivated = false;
 	    instance_ = instance;
 	    _communicator = communicator;
+	    _objectAdapterFactory = objectAdapterFactory;
 	    _servantManager = new IceInternal.ServantManager(instance, name);
-	    _printAdapterReadyDone = false;
+	    _activateOneOffDone = false;
 	    _name = name;
-	    _id = instance.properties().getProperty(name + ".AdapterId");
-	    _replicaGroupId = instance.properties().getProperty(name + ".ReplicaGroupId");
+	    _id = instance.initializationData().properties.getProperty(name + ".AdapterId");
+	    _replicaGroupId = instance.initializationData().properties.getProperty(name + ".ReplicaGroupId");
 	    _incomingConnectionFactories = new ArrayList();
+	    _publishedEndpoints = new ArrayList();
 	    _routerEndpoints = new ArrayList();
-	    _routerInfos = new ArrayList();
+	    _routerInfo = null;
 	    _directCount = 0;
+	    _waitForActivate = false;
 	    _waitForDeactivate = false;
 	    
 	    try
 	    {
-		//
-		// Parse the endpoints, but don't store them in the adapter.
-		// The connection factory might change it, for example, to
-		// fill in the real port number.
-		//
-		ArrayList endpoints = parseEndpoints(endpointInfo);
-		for(int i = 0; i < endpoints.Count; ++i)
+	        if(router == null)
 		{
-		    IceInternal.EndpointI endp = (IceInternal.EndpointI)endpoints[i];
-		    _incomingConnectionFactories.Add(new IceInternal.IncomingConnectionFactory(instance, endp, this));
-		}
-		
-		//
-		// Parse published endpoints. If set, these are used in proxies
-		// instead of the connection factory endpoints.
-		//
-		string endpts = instance_.properties().getProperty(name + ".PublishedEndpoints");
-		_publishedEndpoints = parseEndpoints(endpts);
-		if(_publishedEndpoints.Count == 0)
-		{
-		    foreach(IceInternal.IncomingConnectionFactory factory in _incomingConnectionFactories)
+		    string routerStr = instance_.initializationData().properties.getProperty(name + ".Router");
+		    if(routerStr.Length > 0)
 		    {
-		        _publishedEndpoints.Add(factory.endpoint());
+		        router = RouterPrxHelper.uncheckedCast(instance_.proxyFactory().stringToProxy(routerStr));
 		    }
 		}
-
-		//
-		// Filter out any endpoints that are not meant to be published.
-		//
-		ArrayList tmp = new ArrayList();
-		foreach(IceInternal.EndpointI endpoint in _publishedEndpoints)
+		if(router != null)
 		{
-		    if(endpoint.publish())
+		    _routerInfo = instance_.routerManager().get(router);
+		    if(_routerInfo != null)
 		    {
-		        tmp.Add(endpoint);
+                        //
+                        // Make sure this router is not already registered with another adapter.
+                        //
+                        if(_routerInfo.getAdapter() != null)
+                        {
+			    Ice.AlreadyRegisteredException ex = new Ice.AlreadyRegisteredException();
+			    ex.kindOfObject = "object adapter with router";
+			    ex.id = instance_.identityToString(router.ice_getIdentity());
+			    throw ex;
+                        }
+
+		        //
+		        // Add the router's server proxy endpoints to this object
+		        // adapter.
+		        //
+		        ObjectPrxHelperBase proxy = (ObjectPrxHelperBase)_routerInfo.getServerProxy();
+		        IceInternal.EndpointI[] endpoints = proxy.reference__().getEndpoints();
+		        for(int i = 0; i < endpoints.Length; ++i)
+		        {
+			    _routerEndpoints.Add(endpoints[i]);
+		        }
+		        _routerEndpoints.Sort(); // Must be sorted.
+
+		        //
+		        // Remove duplicate endpoints, so we have a list of unique endpoints.
+		        //
+		        for(int i = 0; i < _routerEndpoints.Count-1;)
+		        {
+			    System.Object o1 = _routerEndpoints[i];
+			    System.Object o2 = _routerEndpoints[i + 1];
+			    if(o1.Equals(o2))
+			    {
+			        _routerEndpoints.RemoveAt(i);
+			    }
+			    else
+			    {
+				++i;
+			    }
+		        }
+
+		        //
+		        // Associate this object adapter with the router. This way,
+		        // new outgoing connections to the router's client proxy will
+		        // use this object adapter for callbacks.
+		        //
+		        _routerInfo.setAdapter(this);
+		    
+		        //
+		        // Also modify all existing outgoing connections to the
+		        // router's client proxy to use this object adapter for
+		        // callbacks.
+		        //      
+		        instance_.outgoingConnectionFactory().setRouterInfo(_routerInfo);
 		    }
 		}
-		_publishedEndpoints = tmp;
-
-		string router = instance_.properties().getProperty(name + ".Router");
-		if(router.Length > 0)
+		else
 		{
-		    addRouter(RouterPrxHelper.uncheckedCast(instance_.proxyFactory().stringToProxy(router)));
-		}
+		    //
+		    // Parse the endpoints, but don't store them in the adapter.
+		    // The connection factory might change it, for example, to
+		    // fill in the real port number.
+		    //
+		    ArrayList endpoints = parseEndpoints(endpointInfo);
+		    for(int i = 0; i < endpoints.Count; ++i)
+		    {
+		        IceInternal.EndpointI endp = (IceInternal.EndpointI)endpoints[i];
+		        _incomingConnectionFactories.Add(
+			    new IceInternal.IncomingConnectionFactory(instance, endp, this, _name));
+		    }
+		    if(endpoints.Count == 0)
+		    {
+		    	IceInternal.TraceLevels tl = instance_.traceLevels();
+			if(tl.network >= 2)
+			{
+			    instance_.initializationData().logger.trace(tl.networkCat,
+								    "created adapter `" + name + "' without endpoints");
+			}
+		    }
 		
-		string locator = instance_.properties().getProperty(name + ".Locator");
+		    //
+		    // Parse published endpoints. If set, these are used in proxies
+		    // instead of the connection factory endpoints.
+		    //
+		    string endpts = instance_.initializationData().properties.getProperty(name + ".PublishedEndpoints");
+		    _publishedEndpoints = parseEndpoints(endpts);
+		    if(_publishedEndpoints.Count == 0)
+		    {
+		        foreach(IceInternal.IncomingConnectionFactory factory in _incomingConnectionFactories)
+		        {
+		            _publishedEndpoints.Add(factory.endpoint());
+		        }
+		    }
+
+		    //
+		    // Filter out any endpoints that are not meant to be published.
+		    //
+		    ArrayList tmp = new ArrayList();
+		    foreach(IceInternal.EndpointI endpoint in _publishedEndpoints)
+		    {
+		        if(endpoint.publish())
+		        {
+		            tmp.Add(endpoint);
+		        }
+		    }
+		    _publishedEndpoints = tmp;
+		}
+
+		string locator = instance_.initializationData().properties.getProperty(name + ".Locator");
 		if(locator.Length > 0)
 		{
 		    setLocator(LocatorPrxHelper.uncheckedCast(instance_.proxyFactory().stringToProxy(locator)));
@@ -903,8 +841,9 @@ namespace Ice
 		
 		if(!instance_.threadPerConnection())
 		{
-		    int size = instance_.properties().getPropertyAsInt(_name + ".ThreadPool.Size");
-		    int sizeMax = instance_.properties().getPropertyAsInt(_name + ".ThreadPool.SizeMax");
+		    int size = instance_.initializationData().properties.getPropertyAsInt(_name + ".ThreadPool.Size");
+		    int sizeMax =
+		        instance_.initializationData().properties.getPropertyAsInt(_name + ".ThreadPool.SizeMax");
 		    if(size > 0 || sizeMax > 0)
 		    {
 			_threadPool = new IceInternal.ThreadPool(instance_, _name + ".ThreadPool", 0);
@@ -928,7 +867,8 @@ namespace Ice
 		{
 		    if(!Environment.HasShutdownStarted)
 		    {
-			instance_.logger().warning("object adapter `" + _name + "' has not been deactivated");
+			instance_.initializationData().logger.warning("object adapter `" + _name +
+								      "' has not been deactivated");
 		    }
 		    else
 		    {
@@ -939,7 +879,7 @@ namespace Ice
 		{
 		    if(!Environment.HasShutdownStarted)
 		    {
-			instance_.logger().warning("object adapter `" + _name +
+			instance_.initializationData().logger.warning("object adapter `" + _name +
 			                           "' deactivation had not been waited for");
 		    }
 		    else
@@ -950,10 +890,11 @@ namespace Ice
 		else
 		{
 		    IceUtil.Assert.FinalizerAssert(_threadPool == null);
-		    IceUtil.Assert.FinalizerAssert(_servantManager == null);
+		    //IceUtil.Assert.FinalizerAssert(_servantManager == null); // Not cleared,it needs to be immutable.
 		    IceUtil.Assert.FinalizerAssert(_communicator == null);
 		    IceUtil.Assert.FinalizerAssert(_incomingConnectionFactories == null);
 		    IceUtil.Assert.FinalizerAssert(_directCount == 0);
+		    IceUtil.Assert.FinalizerAssert(!_waitForActivate);
 		    IceUtil.Assert.FinalizerAssert(!_waitForDeactivate);
 		}
             }   
@@ -1022,7 +963,8 @@ namespace Ice
 		instance_.referenceFactory().create(ident, instance_.getDefaultContext(), facet,
 						    IceInternal.Reference.Mode.ModeTwoway, 
 						    false, id, null, _locatorInfo, 
-						    instance_.defaultsAndOverrides().defaultCollocationOptimization);
+						    instance_.defaultsAndOverrides().defaultCollocationOptimization,
+						    instance_.defaultsAndOverrides().defaultLocatorCacheTimeout);
 	    return instance_.proxyFactory().referenceToProxy(reference);
 	}
 
@@ -1082,14 +1024,21 @@ namespace Ice
 		}
 
 		string s = endpts.Substring(beg, (end) - (beg));
-		IceInternal.EndpointI endp = instance_.endpointFactoryManager().create(s, true);
+		IceInternal.EndpointI endp = instance_.endpointFactoryManager().create(s);
 		if(endp == null)
 		{
+                    if(System.Environment.Version.Major == 1 && s.StartsWith("ssl"))
+                    {
+                        instance_.initializationData().logger.warning(
+                            "SSL endpoint `" + s + "' ignored: IceSSL is not supported with .NET 1.1");
+                        ++end;
+                        continue;
+                    }
 		    Ice.EndpointParseException e2 = new Ice.EndpointParseException();
 		    e2.str = s;
 		    throw e2;
 		}
-		ArrayList endps = endp.expand();
+		ArrayList endps = endp.expand(true);
 		endpoints.AddRange(endps);
 
 		++end;
@@ -1098,6 +1047,102 @@ namespace Ice
 	    return endpoints;
 	}
 
+	private void updateLocatorRegistry(IceInternal.LocatorInfo locatorInfo, ObjectPrx proxy, bool registerProcess)
+	{
+	    if(!registerProcess && _id.Length == 0)
+	    {
+		return; // Nothing to update.
+	    }
+
+	    //
+	    // We must get and call on the locator registry outside the
+	    // thread synchronization to avoid deadlocks. (we can't make
+	    // remote calls within the OA synchronization because the
+	    // remote call will indirectly call isLocal() on this OA with
+	    // the OA factory locked).
+	    //
+	    // TODO: This might throw if we can't connect to the
+	    // locator. Shall we raise a special exception for the
+	    // activate operation instead of a non obvious network
+	    // exception?
+	    //
+	    LocatorRegistryPrx locatorRegistry = locatorInfo != null ? locatorInfo.getLocatorRegistry() : null;
+	    string serverId = "";
+	    if(registerProcess)
+	    {
+		Debug.Assert(instance_ != null);
+		serverId = instance_.initializationData().properties.getProperty("Ice.ServerId");
+
+		if(locatorRegistry == null)
+		{
+		    instance_.initializationData().logger.warning(
+			"object adapter `" + _name + "' cannot register the process without a locator registry");
+		}
+		else if(serverId.Length == 0)
+		{
+		    instance_.initializationData().logger.warning(
+			"object adapter `" + _name + "' cannot register the process without a value for Ice.ServerId");
+		}
+	    }
+
+	    if(locatorRegistry == null)
+	    {
+		return;
+	    }
+
+	    if(_id.Length > 0)
+	    {
+		try
+		{
+		    if(_replicaGroupId.Length == 0)
+		    {
+			locatorRegistry.setAdapterDirectProxy(_id, proxy);
+		    }
+		    else
+		    {
+			locatorRegistry.setReplicatedAdapterDirectProxy(_id, _replicaGroupId, proxy);
+		    }
+		}
+		catch(AdapterNotFoundException)
+		{
+		    NotRegisteredException ex1 = new NotRegisteredException();
+		    ex1.kindOfObject = "object adapter";
+		    ex1.id = _id;
+		    throw ex1;
+		}
+		catch(InvalidReplicaGroupIdException)
+		{
+		    NotRegisteredException ex1 = new NotRegisteredException();
+		    ex1.kindOfObject = "replica group";
+		    ex1.id = _replicaGroupId;
+		    throw ex1;
+		}
+		catch(AdapterAlreadyActiveException)
+		{
+		    ObjectAdapterIdInUseException ex1 = new ObjectAdapterIdInUseException();
+		    ex1.id = _id;
+		    throw ex1;
+		}
+	    }
+	
+	    if(registerProcess && serverId.Length > 0)
+	    {
+		try
+		{
+		    Process servant = new ProcessI(_communicator);
+		    Ice.ObjectPrx process = createDirectProxy(addWithUUID(servant).ice_getIdentity());
+		    locatorRegistry.setServerProcessProxy(serverId, ProcessPrxHelper.uncheckedCast(process));
+		}
+		catch(ServerNotFoundException)
+		{
+		    NotRegisteredException ex1 = new NotRegisteredException();
+		    ex1.id = serverId;
+		    ex1.kindOfObject = "server";
+		    throw ex1;
+		}
+	    }
+	}
+
 	private sealed class ProcessI : ProcessDisp_
 	{
 	    public ProcessI(Communicator communicator)
@@ -1134,18 +1179,20 @@ namespace Ice
 	private bool _deactivated;
 	private IceInternal.Instance instance_;
 	private Communicator _communicator;
+	private IceInternal.ObjectAdapterFactory _objectAdapterFactory;
 	private IceInternal.ThreadPool _threadPool;
 	private IceInternal.ServantManager _servantManager;
-	private bool _printAdapterReadyDone;
+	private bool _activateOneOffDone;
 	private readonly string _name;
 	private readonly string _id;
 	private readonly string _replicaGroupId;
 	private ArrayList _incomingConnectionFactories;
 	private ArrayList _routerEndpoints;
-	private ArrayList _routerInfos;
+	private IceInternal.RouterInfo _routerInfo;
 	private ArrayList _publishedEndpoints;
 	private IceInternal.LocatorInfo _locatorInfo;
 	private int _directCount;
+	private bool _waitForActivate;
 	private bool _waitForDeactivate;
     }
 
