@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -16,9 +16,13 @@ namespace Ice
 {
     public interface ObjectPrx
     {
+	[Obsolete("This method has been deprecated, use ice_getHash instead.")]
         int ice_hash();
+        int ice_getHash();
 
+	[Obsolete("This method has been deprecated, use ice_getCommunicator instead.")]
 	Communicator ice_communicator();
+	Communicator ice_getCommunicator();
 
 	string ice_toString();
 
@@ -44,20 +48,53 @@ namespace Ice
             Context context);
 
         Identity ice_getIdentity();
+	[Obsolete("This method has been deprecated, use ice_identity instead.")]
         ObjectPrx ice_newIdentity(Identity newIdentity);
-
+        ObjectPrx ice_identity(Identity newIdentity);
+	
         Context ice_getContext();
+	[Obsolete("This method has been deprecated, use ice_context instead.")]
         ObjectPrx ice_newContext(Context newContext);
+        ObjectPrx ice_context(Context newContext);
         ObjectPrx ice_defaultContext();
-
+	
         string ice_getFacet();
+	[Obsolete("This method has been deprecated, use ice_facet instead.")]
         ObjectPrx ice_newFacet(string newFacet);
+        ObjectPrx ice_facet(string newFacet);
 
         string ice_getAdapterId();
+	[Obsolete("This method has been deprecated, use ice_adapterId instead.")]
         ObjectPrx ice_newAdapterId(string newAdapterId);
+        ObjectPrx ice_adapterId(string newAdapterId);
 
 	Endpoint[] ice_getEndpoints();
+	[Obsolete("This method has been deprecated, use ice_endpoints instead.")]
         ObjectPrx ice_newEndpoints(Endpoint[] newEndpoints);
+        ObjectPrx ice_endpoints(Endpoint[] newEndpoints);
+
+	int ice_getLocatorCacheTimeout();
+	ObjectPrx ice_locatorCacheTimeout(int timeout);
+	
+	bool ice_isConnectionCached();
+	ObjectPrx ice_connectionCached(bool newCache);
+
+	EndpointSelectionType ice_getEndpointSelection();
+	ObjectPrx ice_endpointSelection(EndpointSelectionType newType);
+
+	bool ice_isSecure();
+        ObjectPrx ice_secure(bool b);
+
+	Ice.RouterPrx ice_getRouter();
+        ObjectPrx ice_router(Ice.RouterPrx router);
+
+	Ice.LocatorPrx ice_getLocator();
+        ObjectPrx ice_locator(Ice.LocatorPrx locator);
+
+	bool ice_isCollocationOptimized();
+	[Obsolete("This method has been deprecated, use ice_collocationOptimized instead.")]
+        ObjectPrx ice_collocationOptimization(bool b);
+        ObjectPrx ice_collocationOptimized(bool b);
 
         ObjectPrx ice_twoway();
         bool ice_isTwoway();
@@ -70,15 +107,13 @@ namespace Ice
         ObjectPrx ice_batchDatagram();
         bool ice_isBatchDatagram();
 
-        ObjectPrx ice_secure(bool b);
         ObjectPrx ice_compress(bool co);
         ObjectPrx ice_timeout(int t);
         ObjectPrx ice_connectionId(string connectionId);
-        ObjectPrx ice_router(Ice.RouterPrx router);
-        ObjectPrx ice_locator(Ice.LocatorPrx locator);
-        ObjectPrx ice_collocationOptimization(bool b);
 
+	[Obsolete("This method has been deprecated, use ice_getConnection instead.")]
         Connection ice_connection();
+        Connection ice_getConnection();
     }
 
     public class ObjectPrxHelperBase : ObjectPrx
@@ -89,11 +124,21 @@ namespace Ice
         }
 
         public int ice_hash()
+        {
+            return ice_getHash();
+        }
+
+        public int ice_getHash()
         {
             return _reference.GetHashCode();
         }
 
         public Communicator ice_communicator()
+	{
+	    return ice_getCommunicator();
+	}
+
+        public Communicator ice_getCommunicator()
 	{
 	    return _reference.getCommunicator();
 	}
@@ -124,9 +169,9 @@ namespace Ice
                     ObjectDel_ del__ = getDelegate__();
                     return del__.ice_isA(id__, context__);
                 }
-                catch(IceInternal.NonRepeatable ex__)
+                catch(IceInternal.LocalExceptionWrapper ex__)
                 {
-                    cnt__ = handleException__(ex__.get(), cnt__);
+                    cnt__ = handleExceptionWrapperRelaxed__(ex__, cnt__);
                 }
                 catch(LocalException ex__)
                 {
@@ -152,9 +197,9 @@ namespace Ice
                     del__.ice_ping(context__);
                     return;
                 }
-                catch(IceInternal.NonRepeatable ex__)
+                catch(IceInternal.LocalExceptionWrapper ex__)
                 {
-                    cnt__ = handleException__(ex__.get(), cnt__);
+		    cnt__ = handleExceptionWrapperRelaxed__(ex__, cnt__);
                 }
                 catch(LocalException ex__)
                 {
@@ -179,9 +224,9 @@ namespace Ice
                     ObjectDel_ del__ = getDelegate__();
                     return del__.ice_ids(context__);
                 }
-                catch(IceInternal.NonRepeatable ex__)
+                catch(IceInternal.LocalExceptionWrapper ex__)
                 {
-                    cnt__ = handleException__(ex__.get(), cnt__);
+		    cnt__ = handleExceptionWrapperRelaxed__(ex__, cnt__);
                 }
                 catch(LocalException ex__)
                 {
@@ -206,9 +251,9 @@ namespace Ice
                     ObjectDel_ del__ = getDelegate__();
                     return del__.ice_id(context__);
                 }
-                catch(IceInternal.NonRepeatable ex__)
+                catch(IceInternal.LocalExceptionWrapper ex__)
                 {
-                    cnt__ = handleException__(ex__.get(), cnt__);
+		    cnt__ = handleExceptionWrapperRelaxed__(ex__, cnt__);
                 }
                 catch(LocalException ex__)
                 {
@@ -233,15 +278,15 @@ namespace Ice
                     ObjectDel_ del__ = getDelegate__();
                     return del__.ice_invoke(operation, mode, inParams, out outParams, context);
                 }
-                catch(IceInternal.NonRepeatable ex__)
+                catch(IceInternal.LocalExceptionWrapper ex__)
                 {
                     if(mode == OperationMode.Nonmutating || mode == OperationMode.Idempotent)
                     {
-                        cnt__ = handleException__(ex__.get(), cnt__);
+                        cnt__ = handleExceptionWrapperRelaxed__(ex__, cnt__);
                     }
                     else
                     {
-                        rethrowException__(ex__.get());
+                        handleExceptionWrapper__(ex__);
                     }
                 }
                 catch(LocalException ex__)
@@ -268,8 +313,12 @@ namespace Ice
             return _reference.getIdentity();
         }
 
-        public ObjectPrx ice_newIdentity(Identity newIdentity)
+        public ObjectPrx ice_identity(Identity newIdentity)
         {
+	    if(newIdentity.name.Equals(""))
+	    {
+	        throw new IllegalIdentityException();
+	    }
             if(newIdentity.Equals(_reference.getIdentity()))
             {
                 return this;
@@ -282,18 +331,28 @@ namespace Ice
             }
         }
 
+        public ObjectPrx ice_newIdentity(Identity newIdentity)
+        {
+	    return ice_identity(newIdentity);
+	}
+
         public Context ice_getContext()
         {
             return _reference.getContext();
         }
 
-        public ObjectPrx ice_newContext(Context newContext)
+        public ObjectPrx ice_context(Context newContext)
         {
             ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
             proxy.setup(_reference.changeContext(newContext));
             return proxy;
         }
 
+        public ObjectPrx ice_newContext(Context newContext)
+        {
+	    return ice_context(newContext);
+	}
+
 	public ObjectPrx ice_defaultContext()
 	{
             ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
@@ -306,7 +365,7 @@ namespace Ice
             return _reference.getFacet();
         }
 
-        public ObjectPrx ice_newFacet(string newFacet)
+        public ObjectPrx ice_facet(string newFacet)
         {
             if(newFacet == null)
             {
@@ -325,12 +384,17 @@ namespace Ice
             }
         }
 
+        public ObjectPrx ice_newFacet(string newFacet)
+        {
+	    return ice_facet(newFacet);
+	}
+
         public string ice_getAdapterId()
         {
             return _reference.getAdapterId();
         }
 
-        public ObjectPrx ice_newAdapterId(string newAdapterId)
+        public ObjectPrx ice_adapterId(string newAdapterId)
         {
             if(newAdapterId == null)
             {
@@ -349,12 +413,17 @@ namespace Ice
             }
         }
 
+        public ObjectPrx ice_newAdapterId(string newAdapterId)
+        {
+	    return ice_adapterId(newAdapterId);
+	}
+
         public Endpoint[] ice_getEndpoints()
         {
             return _reference.getEndpoints();
         }
 
-        public ObjectPrx ice_newEndpoints(Endpoint[] newEndpoints)
+        public ObjectPrx ice_endpoints(Endpoint[] newEndpoints)
         {
             if(Arrays.Equals(newEndpoints, _reference.getEndpoints()))
             {
@@ -363,14 +432,103 @@ namespace Ice
             else
             {
                 ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(_reference.changeEndpoints((IceInternal.EndpointI[])newEndpoints));
+		ArrayList arr = ArrayList.Adapter(newEndpoints);
+		IceInternal.EndpointI[] endpts = (IceInternal.EndpointI[])arr.ToArray(typeof(IceInternal.EndpointI));
+                proxy.setup(_reference.changeEndpoints(endpts));
                 return proxy;
             }
         }
 
-        public ObjectPrx ice_twoway()
+        public ObjectPrx ice_newEndpoints(Endpoint[] newEndpoints)
         {
-            IceInternal.Reference @ref = _reference.changeMode(IceInternal.Reference.Mode.ModeTwoway);
+	    return ice_endpoints(newEndpoints);
+	}
+
+        public int ice_getLocatorCacheTimeout()
+        {
+            return _reference.getLocatorCacheTimeout();
+        }
+
+        public ObjectPrx ice_locatorCacheTimeout(int newTimeout)
+        {
+            if(newTimeout == _reference.getLocatorCacheTimeout())
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeLocatorCacheTimeout(newTimeout));
+                return proxy;
+            }
+        }
+
+        public bool ice_isConnectionCached()
+        {
+            return _reference.getCacheConnection();
+        }
+
+        public ObjectPrx ice_connectionCached(bool newCache)
+        {
+            if(newCache == _reference.getCacheConnection())
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeCacheConnection(newCache));
+                return proxy;
+            }
+        }
+
+        public EndpointSelectionType ice_getEndpointSelection()
+        {
+            return _reference.getEndpointSelection();
+        }
+
+        public ObjectPrx ice_endpointSelection(EndpointSelectionType newType)
+        {
+            if(newType == _reference.getEndpointSelection())
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeEndpointSelection(newType));
+                return proxy;
+            }
+        }
+
+        public bool ice_isSecure()
+        {
+            return _reference.getSecure();
+        }
+
+        public ObjectPrx ice_secure(bool b)
+        {
+            if(b == _reference.getSecure())
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeSecure(b));
+                return proxy;
+            }
+        }
+
+        public Ice.RouterPrx ice_getRouter()
+        {
+	    IceInternal.RouterInfo ri = _reference.getRouterInfo();
+	    return ri != null ? ri.getRouter() : null;
+        }
+
+        public ObjectPrx ice_router(RouterPrx router)
+        {
+            IceInternal.Reference @ref = _reference.changeRouter(router);
             if(@ref.Equals(_reference))
             {
                 return this;
@@ -383,6 +541,65 @@ namespace Ice
             }
         }
 
+        public Ice.LocatorPrx ice_getLocator()
+        {
+	    IceInternal.LocatorInfo li = _reference.getLocatorInfo();
+	    return li != null ? li.getLocator() : null;
+        }
+
+        public ObjectPrx ice_locator(LocatorPrx locator)
+        {
+            IceInternal.Reference @ref = _reference.changeLocator(locator);
+            if(@ref.Equals(_reference))
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(@ref);
+                return proxy;
+            }
+        }
+
+        public bool ice_isCollocationOptimized()
+        {
+            return _reference.getCollocationOptimization();
+        }
+
+        public ObjectPrx ice_collocationOptimization(bool b)
+        {
+	    return ice_collocationOptimized(b);
+	}
+
+        public ObjectPrx ice_collocationOptimized(bool b)
+        {
+            if(b == _reference.getCollocationOptimization())
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeCollocationOptimization(b));
+                return proxy;
+            }
+        }
+
+        public ObjectPrx ice_twoway()
+        {
+            if(_reference.getMode() == IceInternal.Reference.Mode.ModeTwoway)
+            {
+                return this;
+            }
+            else
+            {
+                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
+                proxy.setup(_reference.changeMode(IceInternal.Reference.Mode.ModeTwoway));
+                return proxy;
+            }
+        }
+
         public bool ice_isTwoway()
         {
             return _reference.getMode() == IceInternal.Reference.Mode.ModeTwoway;
@@ -390,15 +607,14 @@ namespace Ice
 
         public ObjectPrx ice_oneway()
         {
-            IceInternal.Reference @ref = _reference.changeMode(IceInternal.Reference.Mode.ModeOneway);
-            if(@ref.Equals(_reference))
+            if(_reference.getMode() == IceInternal.Reference.Mode.ModeOneway)
             {
                 return this;
             }
             else
             {
                 ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
+                proxy.setup(_reference.changeMode(IceInternal.Reference.Mode.ModeOneway));
                 return proxy;
             }
         }
@@ -410,15 +626,14 @@ namespace Ice
 
         public ObjectPrx ice_batchOneway()
         {
-            IceInternal.Reference @ref = _reference.changeMode(IceInternal.Reference.Mode.ModeBatchOneway);
-            if(@ref.Equals(_reference))
+            if(_reference.getMode() == IceInternal.Reference.Mode.ModeBatchOneway)
             {
                 return this;
             }
             else
             {
                 ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
+                proxy.setup(_reference.changeMode(IceInternal.Reference.Mode.ModeBatchOneway));
                 return proxy;
             }
         }
@@ -430,15 +645,14 @@ namespace Ice
 
         public ObjectPrx ice_datagram()
         {
-            IceInternal.Reference @ref = _reference.changeMode(IceInternal.Reference.Mode.ModeDatagram);
-            if(@ref.Equals(_reference))
+            if(_reference.getMode() == IceInternal.Reference.Mode.ModeDatagram)
             {
                 return this;
             }
             else
             {
                 ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
+                proxy.setup(_reference.changeMode(IceInternal.Reference.Mode.ModeDatagram));
                 return proxy;
             }
         }
@@ -450,15 +664,14 @@ namespace Ice
 
         public ObjectPrx ice_batchDatagram()
         {
-            IceInternal.Reference @ref = _reference.changeMode(IceInternal.Reference.Mode.ModeBatchDatagram);
-            if(@ref.Equals(_reference))
+            if(_reference.getMode() == IceInternal.Reference.Mode.ModeBatchDatagram)
             {
                 return this;
             }
             else
             {
                 ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
+                proxy.setup(_reference.changeMode(IceInternal.Reference.Mode.ModeBatchDatagram));
                 return proxy;
             }
         }
@@ -468,21 +681,6 @@ namespace Ice
             return _reference.getMode() == IceInternal.Reference.Mode.ModeBatchDatagram;
         }
 
-        public ObjectPrx ice_secure(bool b)
-        {
-            IceInternal.Reference @ref = _reference.changeSecure(b);
-            if(@ref.Equals(_reference))
-            {
-                return this;
-            }
-            else
-            {
-                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
-                return proxy;
-            }
-        }
-
         public ObjectPrx ice_compress(bool co)
         {
             IceInternal.Reference @ref = _reference.changeCompress(co);
@@ -528,52 +726,12 @@ namespace Ice
             }
         }
 
-        public ObjectPrx ice_router(RouterPrx router)
-        {
-            IceInternal.Reference @ref = _reference.changeRouter(router);
-            if(@ref.Equals(_reference))
-            {
-                return this;
-            }
-            else
-            {
-                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
-                return proxy;
-            }
-        }
-
-        public ObjectPrx ice_locator(LocatorPrx locator)
-        {
-            IceInternal.Reference @ref = _reference.changeLocator(locator);
-            if(@ref.Equals(_reference))
-            {
-                return this;
-            }
-            else
-            {
-                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
-                return proxy;
-            }
-        }
-
-        public ObjectPrx ice_collocationOptimization(bool b)
-        {
-            IceInternal.Reference @ref = _reference.changeCollocationOptimization(b);
-            if(@ref.Equals(_reference))
-            {
-                return this;
-            }
-            else
-            {
-                ObjectPrxHelperBase proxy = new ObjectPrxHelperBase();
-                proxy.setup(@ref);
-                return proxy;
-            }
-        }
-
         public Connection ice_connection()
+        {
+	    return ice_getConnection();
+	}
+
+        public Connection ice_getConnection()
         {
             int cnt__ = 0;
             while(true)
@@ -581,7 +739,8 @@ namespace Ice
                 try
                 {
                     ObjectDel_ del__ = getDelegate__();
-                    return del__.ice_connection();
+		    bool comp;
+                    return del__.getConnection__(out comp);
                 }
                 catch(LocalException ex__)
                 {
@@ -640,18 +799,27 @@ namespace Ice
 
             _reference = @ref;
 
-            if(delegateD != null)
-            {
-                ObjectDelD_ @delegate = createDelegateD__();
-                @delegate.copyFrom__(delegateD);
-                _delegate = @delegate;
-            }
-            else if(delegateM != null)
-            {
-                ObjectDelM_ @delegate = createDelegateM__();
-                @delegate.copyFrom__(delegateM);
-                _delegate = @delegate;
-            }
+	    if(_reference.getCacheConnection())
+	    {
+		//
+		// The _delegate attribute is only used if "cache connection"
+		// is enabled. If it's not enabled, we don't keep track of the
+		// delegate -- a new delegate is created for each invocations.
+		//	
+
+		if(delegateD != null)
+		{
+		    ObjectDelD_ @delegate = createDelegateD__();
+		    @delegate.copyFrom__(delegateD);
+		    _delegate = @delegate;
+		}
+		else if(delegateM != null)
+		{
+		    ObjectDelM_ @delegate = createDelegateM__();
+		    @delegate.copyFrom__(delegateM);
+		    _delegate = @delegate;
+		}
+	    }
         }
 
         public int handleException__(LocalException ex, int cnt)
@@ -679,12 +847,32 @@ namespace Ice
             }
         }
 
-        public void rethrowException__(LocalException ex)
+        public void handleExceptionWrapper__(IceInternal.LocalExceptionWrapper ex)
         {
             lock(this)
             {
                 _delegate = null;
-                throw ex;
+            }
+
+            if(!ex.retry())
+            {
+                throw ex.get();
+            }
+        }
+
+        public int handleExceptionWrapperRelaxed__(IceInternal.LocalExceptionWrapper ex, int cnt)
+        {
+            if(!ex.retry())
+            {
+                return handleException__(ex.get(), cnt);
+            }
+            else
+            {
+                lock(this)
+                {
+                    _delegate = null;
+                }
+                return cnt;
             }
         }
 
@@ -707,46 +895,53 @@ namespace Ice
         {
             lock(this)
             {
-                if(_delegate == null)
+                if(_delegate != null)
                 {
-                    if(_reference.getCollocationOptimization())
-                    {
-                        ObjectAdapter adapter = _reference.getInstance().objectAdapterFactory().findObjectAdapter(this);
-                        if(adapter != null)
-                        {
-                            ObjectDelD_ @delegate = createDelegateD__();
-                            @delegate.setup(_reference, adapter);
-                            _delegate = @delegate;
-                        }
-                    }
+		    return _delegate;
+		}
 
-                    if(_delegate == null)
-                    {
-                        ObjectDelM_ @delegate = createDelegateM__();
-                        @delegate.setup(_reference);
-                        _delegate = @delegate;
-
-                        //
-                        // If this proxy is for a non-local object, and we are
-                        // using a router, then add this proxy to the router info
-                        // object.
-                        //
-                        try
-                        {
-                            IceInternal.RoutableReference rr = (IceInternal.RoutableReference)_reference;
-                            if(rr != null && rr.getRouterInfo() != null)
-                            {
-                                rr.getRouterInfo().addProxy(this);
-                            }
-                        }
-                        catch(InvalidCastException)
-                        {
-                        }
-                    }
-                }
-
-                return _delegate;
-            }
+		ObjectDel_ @delegate = null;
+		if(_reference.getCollocationOptimization())
+		{
+		    ObjectAdapter adapter = _reference.getInstance().objectAdapterFactory().findObjectAdapter(this);
+		    if(adapter != null)
+		    {
+			ObjectDelD_ d = createDelegateD__();
+			d.setup(_reference, adapter);
+			@delegate = d;
+		    }
+		}
+		
+		if(@delegate == null)
+		{
+		    ObjectDelM_ d = createDelegateM__();
+		    d.setup(_reference);
+		    @delegate = d;
+		    
+		    //
+		    // If this proxy is for a non-local object, and we are
+		    // using a router, then add this proxy to the router info
+		    // object.
+		    //
+		    IceInternal.RouterInfo ri = _reference.getRouterInfo();
+		    if(ri != null)
+		    {
+			ri.addProxy(this);
+		    }
+		}
+	    
+		if(_reference.getCacheConnection())
+		{
+		    //
+		    // The _delegate attribute is only used if "cache connection"
+		    // is enabled. If it's not enabled, we don't keep track of the
+		    // delegate -- a new delegate is created for each invocations.
+		    //	
+		    _delegate = @delegate;
+		}
+		
+		return @delegate;
+	    }
         }
 
         protected virtual ObjectDelM_ createDelegateM__()
@@ -803,7 +998,7 @@ namespace Ice
             {
                 try
                 {
-                    Ice.ObjectPrx bb = b.ice_newFacet(f);
+                    Ice.ObjectPrx bb = b.ice_facet(f);
                     bool ok = bb.ice_isA("::Object");
                     Debug.Assert(ok);
                     ObjectPrxHelper h = new ObjectPrxHelper();
@@ -824,7 +1019,7 @@ namespace Ice
             {
                 try
                 {
-                    Ice.ObjectPrx bb = b.ice_newFacet(f);
+                    Ice.ObjectPrx bb = b.ice_facet(f);
                     bool ok = bb.ice_isA("::Object", ctx);
                     Debug.Assert(ok);
                     ObjectPrxHelper h = new ObjectPrxHelper();
@@ -848,7 +1043,7 @@ namespace Ice
             ObjectPrx d = null;
             if(b != null)
             {
-                Ice.ObjectPrx bb = b.ice_newFacet(f);
+                Ice.ObjectPrx bb = b.ice_facet(f);
                 ObjectPrxHelper h = new ObjectPrxHelper();
                 h.copyFrom__(bb);
                 d = h;
@@ -865,7 +1060,8 @@ namespace Ice
 	string ice_id(Ice.Context context);
 	bool ice_invoke(string operation, Ice.OperationMode mode, byte[] inParams, out byte[] outParams,
 			Ice.Context context);
-        Connection ice_connection();
+
+        ConnectionI getConnection__(out bool compress);
     }
 
     public class ObjectDelD_ : ObjectDel_
@@ -949,7 +1145,7 @@ namespace Ice
             throw new CollocationOptimizationException();
         }
 	
-        public virtual Connection ice_connection()
+        public virtual ConnectionI getConnection__(out bool compress)
         {
             throw new CollocationOptimizationException();
         }
@@ -988,6 +1184,7 @@ namespace Ice
             current.operation = op;
             current.mode = mode;
             current.ctx = context;
+	    current.requestId = -1;
         }
 	
         public virtual void setup(IceInternal.Reference rf, Ice.ObjectAdapter adapter)
@@ -1040,7 +1237,7 @@ namespace Ice
 		}
 		catch(LocalException ex__)
 		{
-		    throw new IceInternal.NonRepeatable(ex__);
+		    throw new IceInternal.LocalExceptionWrapper(ex__, false);
 		}
             }
             finally
@@ -1072,7 +1269,7 @@ namespace Ice
 		}
 		catch(LocalException ex__)
 		{
-		    throw new IceInternal.NonRepeatable(ex__);
+		    throw new IceInternal.LocalExceptionWrapper(ex__, false);
 		}
             }
             finally
@@ -1105,7 +1302,7 @@ namespace Ice
 		}
 		catch(LocalException ex__)
 		{
-		    throw new IceInternal.NonRepeatable(ex__);
+		    throw new IceInternal.LocalExceptionWrapper(ex__, false);
 		}
             }
             finally
@@ -1138,7 +1335,7 @@ namespace Ice
 		}
 		catch(LocalException ex__)
 		{
-		    throw new IceInternal.NonRepeatable(ex__);
+		    throw new IceInternal.LocalExceptionWrapper(ex__, false);
 		}
             }
             finally
@@ -1174,7 +1371,7 @@ namespace Ice
                     }
                     catch(LocalException ex__)
                     {
-                        throw new IceInternal.NonRepeatable(ex__);
+                        throw new IceInternal.LocalExceptionWrapper(ex__, false);
                     }
                 }
                 return ok;
@@ -1185,8 +1382,9 @@ namespace Ice
             }
         }
 	
-        public virtual Connection ice_connection()
+        public virtual ConnectionI getConnection__(out bool compress)
         {
+	    compress = compress__;
             return connection__;
         }
 	
