@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -61,10 +61,24 @@ namespace IceInternal
 	{
 	    return communicator_;
 	}
+
+	public virtual RouterInfo getRouterInfo()
+	{
+	    return null;
+	}
+
+	public virtual LocatorInfo getLocatorInfo()
+	{
+	    return null;
+	}
+
 	public abstract bool getSecure();
 	public abstract string getAdapterId();
 	public abstract EndpointI[] getEndpoints();
 	public abstract bool getCollocationOptimization();
+	public abstract int getLocatorCacheTimeout();
+	public abstract bool getCacheConnection();
+	public abstract Ice.EndpointSelectionType getEndpointSelection();
 
 	//
 	// The change* methods (here and in derived classes) create
@@ -131,6 +145,9 @@ namespace IceInternal
 	public abstract Reference changeCollocationOptimization(bool newCollocationOptimization);
 	public abstract Reference changeAdapterId(string newAdapterId);
 	public abstract Reference changeEndpoints(EndpointI[] newEndpoints);
+	public abstract Reference changeLocatorCacheTimeout(int newTimeout);
+	public abstract Reference changeCacheConnection(bool newCache);
+	public abstract Reference changeEndpointSelection(Ice.EndpointSelectionType newType);
 
 	public override int GetHashCode()
 	{
@@ -207,6 +224,13 @@ namespace IceInternal
 	//
 	public override string ToString()
 	{
+	    //
+	    // WARNING: Certain features, such as proxy validation in Glacier2,
+	    // depend on the format of proxy strings. Changes to toString() and
+	    // methods called to generate parts of the reference string could break
+	    // these features. Please review for all features that depend on the
+	    // format of proxyToString() before changing this and related code.
+	    //
 	    StringBuilder s = new StringBuilder();
 
 	    //
@@ -214,7 +238,7 @@ namespace IceInternal
 	    // the reference parser uses as separators, then we enclose
 	    // the identity string in quotes.
 	    //
-	    string id = Ice.Util.identityToString(identity_);
+	    string id = instance_.identityToString(identity_);
 	    if(IceUtil.StringUtil.findFirstOf(id, " \t\n\r:@") != -1)
 	    {
 		s.Append('"');
@@ -249,31 +273,31 @@ namespace IceInternal
 
 	    switch(mode_)
 	    {
-		case Mode.ModeTwoway:
+	    case Mode.ModeTwoway:
 		{
 		    s.Append(" -t");
 		    break;
 		}
 
-		case Mode.ModeOneway:
+	    case Mode.ModeOneway:
 		{
 		    s.Append(" -o");
 		    break;
 		}
 
-		case Mode.ModeBatchOneway:
+	    case Mode.ModeBatchOneway:
 		{
 		    s.Append(" -O");
 		    break;
 		}
 
-		case Mode.ModeDatagram:
+	    case Mode.ModeDatagram:
 		{
 		    s.Append(" -d");
 		    break;
 		}
 
-		case Mode.ModeBatchDatagram:
+	    case Mode.ModeBatchDatagram:
 		{
 		    s.Append(" -D");
 		    break;
@@ -365,253 +389,7 @@ namespace IceInternal
 	    hashInitialized_ = false;
 	}
 
-	//
-	// Filter endpoints based on criteria from this reference.
-	//
-	protected EndpointI[] filterEndpoints(EndpointI[] allEndpoints)
-	{
-	    ArrayList endpoints = new ArrayList();
-
-	    //
-	    // Filter out unknown endpoints.
-	    //
-	    for(int i = 0; i < allEndpoints.Length; i++)
-	    {
-		if(!allEndpoints[i].unknown())
-		{
-		    endpoints.Add(allEndpoints[i]);
-		}
-	    }
-
-	    switch(mode_)
-	    {
-		case Reference.Mode.ModeTwoway:
-		case Reference.Mode.ModeOneway:
-		case Reference.Mode.ModeBatchOneway:
-		{
-		    //
-		    // Filter out datagram endpoints.
-		    //
-		    ArrayList tmp = new ArrayList();
-		    foreach(EndpointI endpoint in endpoints)
-		    {
-		    	if(!endpoint.datagram())
-			{
-			    tmp.Add(endpoint);
-			}
-		    }
-		    endpoints = tmp;
-		    break;
-		}
-
-		case Reference.Mode.ModeDatagram:
-		case Reference.Mode.ModeBatchDatagram:
-		{
-		    //
-		    // Filter out non-datagram endpoints.
-		    //
-		    ArrayList tmp = new ArrayList();
-		    foreach(EndpointI endpoint in endpoints)
-		    {
-		    	if(endpoint.datagram())
-			{
-			    tmp.Add(endpoint);
-			}
-		    }
-		    endpoints = tmp;
-		    break;
-		}
-	    }
-
-	    //
-	    // Randomize the order of endpoints.
-	    //
-	    for(int i = 0; i < endpoints.Count - 2; ++i)
-	    {
-		int r = _rand.Next(endpoints.Count - i) + i;
-		Debug.Assert(r >= i && r < endpoints.Count);
-		if(r != i)
-		{
-		    object tmp = endpoints[i];
-		    endpoints[i] = endpoints[r];
-		    endpoints[r] = tmp;
-		}
-	    }
-
-	    //
-	    // If a secure connection is requested, remove all non-secure
-	    // endpoints. Otherwise make non-secure endpoints preferred over
-	    // secure endpoints by partitioning the endpoint vector, so that
-	    // non-secure endpoints come first.
-	    //
-	    if(getSecure())
-	    {
-		ArrayList tmp = new ArrayList();
-		foreach(EndpointI endpoint in endpoints)
-		{
-		    if(endpoint.secure())
-		    {
-			tmp.Add(endpoint);
-		    }
-		}
-		endpoints = tmp;
-	    }
-	    else
-	    {
-		endpoints.Sort(_endpointComparator);
-	    }
-	    
-	    EndpointI[] arr = new EndpointI[endpoints.Count];
-	    if(arr.Length != 0)
-	    {
-		endpoints.CopyTo(arr);
-	    }
-	    return arr;
-	}
-
-	private class EndpointComparator : IComparer
-	{
-	    public int Compare(object l, object r)
-	    {
-		IceInternal.EndpointI le = (IceInternal.EndpointI)l;
-		IceInternal.EndpointI re = (IceInternal.EndpointI)r;
-		bool ls = le.secure();
-		bool rs = re.secure();
-		if((ls && rs) || (!ls && !rs))
-		{
-		    return 0;
-		}
-		else if(!ls && rs)
-		{
-		    return -1;
-		}
-		else
-		{
-		    return 1;
-		}
-	    }
-	}
-	
-	private static EndpointComparator _endpointComparator = new EndpointComparator();
-
-	//
-	// Filter connections based on criteria from this reference.
-	//
-	public Ice.ConnectionI[]
-	filterConnections(Ice.ConnectionI[] allConnections)
-	{
-	    ArrayList connections = new ArrayList();
-
-	    switch(mode_)
-	    {
-		case Reference.Mode.ModeTwoway:
-		case Reference.Mode.ModeOneway:
-		case Reference.Mode.ModeBatchOneway:
-		{
-		    //
-		    // Filter out datagram endpoints.
-		    //
-		    for(int i = 0; i < allConnections.Length; ++i)
-		    {
-			if(!allConnections[i].endpoint().datagram())
-			{
-			    connections.Add(allConnections[i]);
-			}
-		    }
-
-		    break;
-		}
-
-		case Reference.Mode.ModeDatagram:
-		case Reference.Mode.ModeBatchDatagram:
-		{
-		    //
-		    // Filter out non-datagram endpoints.
-		    //
-		    for(int i = 0; i < allConnections.Length; ++i)
-		    {
-			if(allConnections[i].endpoint().datagram())
-			{
-			    connections.Add(allConnections[i]);
-			}
-		    }
-
-		    break;
-		}
-	    }
-
-	    //
-	    // Randomize the order of connections.
-	    //
-	    for(int i = 0; i < connections.Count - 2; ++i)
-	    {
-		int r = _rand.Next(connections.Count - i) + i;
-		Debug.Assert(r >= i && r < connections.Count);
-		if(r != i)
-		{
-		    object tmp = connections[i];
-		    connections[i] = connections[r];
-		    connections[r] = tmp;
-		}
-	    }
-
-	    //
-	    // If a secure connection is requested, remove all non-secure
-	    // endpoints. Otherwise make non-secure endpoints preferred over
-	    // secure endpoints by partitioning the endpoint vector, so that
-	    // non-secure endpoints come first.
-	    //
-	    if(getSecure())
-	    {
-		ArrayList tmp = new ArrayList();
-		foreach(Ice.ConnectionI connection in connections)
-		{
-		    if(connection.endpoint().secure())
-		    {
-			tmp.Add(connection);
-		    }
-		}
-		connections = tmp;
-	    }
-	    else
-	    {
-		connections.Sort(_connectionComparator);
-	    }
-	    
-	    Ice.ConnectionI[] arr = new Ice.ConnectionI[connections.Count];
-	    if(arr.Length != 0)
-	    {
-		connections.CopyTo(arr);
-	    }
-	    return arr;
-	}
-
-	private class ConnectionComparator : IComparer
-	{
-	    public int Compare(object l, object r)
-	    {
-		Ice.ConnectionI lc = (Ice.ConnectionI)l;
-		Ice.ConnectionI rc = (Ice.ConnectionI)r;
-		bool ls = lc.endpoint().secure();
-		bool rs = rc.endpoint().secure();
-		if((ls && rs) || (!ls && !rs))
-		{
-		    return 0;
-		}
-		else if(!ls && rs)
-		{
-		    return -1;
-		}
-		else
-		{
-		    return 1;
-		}
-	    }
-	}
-	
-	private static ConnectionComparator _connectionComparator = new ConnectionComparator();
-
-	private static System.Random _rand = new System.Random(unchecked((int)System.DateTime.Now.Ticks));
+	protected static System.Random rand_ = new System.Random(unchecked((int)System.DateTime.Now.Ticks));
     }
 
     public class FixedReference : Reference
@@ -653,6 +431,21 @@ namespace IceInternal
 	    return false;
 	}
 
+	public override int getLocatorCacheTimeout()
+        {
+	    return 0;
+	}
+
+	public override bool getCacheConnection()
+	{
+	    return false;
+	}
+
+	public override Ice.EndpointSelectionType getEndpointSelection()
+	{
+	    return Ice.EndpointSelectionType.Random;
+	}
+
 	public override Reference changeSecure(bool sec)
 	{
 	    return this;
@@ -683,6 +476,21 @@ namespace IceInternal
 	    return this;
 	}
 
+	public override Reference changeLocatorCacheTimeout(int newTimeout)
+        {
+	    return this;
+	}
+
+	public override Reference changeCacheConnection(bool newCache)
+	{
+	    return this;
+	}
+
+	public override Reference changeEndpointSelection(Ice.EndpointSelectionType newType)
+	{
+	    return this;
+	}
+
 	public override Reference changeCompress(bool newCompress)
 	{
 	    // TODO: FixedReferences should probably have a _compress flag,
@@ -727,6 +535,7 @@ namespace IceInternal
 
 	    Ice.ConnectionI connection = filteredConns[0];
 	    Debug.Assert(connection != null);
+	    connection.throwException(); // Throw in case our connection is already destroyed.
 	    compress = connection.endpoint().compress();
 
 	    return connection;
@@ -758,12 +567,128 @@ namespace IceInternal
             return base.GetHashCode();
         }
 
+	//
+	// Filter connections based on criteria from this reference.
+	//
+	public Ice.ConnectionI[]
+	filterConnections(Ice.ConnectionI[] allConnections)
+	{
+	    ArrayList connections = new ArrayList();
+
+	    switch(getMode())
+	    {
+		case Reference.Mode.ModeTwoway:
+		case Reference.Mode.ModeOneway:
+		case Reference.Mode.ModeBatchOneway:
+		{
+		    //
+		    // Filter out datagram endpoints.
+		    //
+		    for(int i = 0; i < allConnections.Length; ++i)
+		    {
+			if(!allConnections[i].endpoint().datagram())
+			{
+			    connections.Add(allConnections[i]);
+			}
+		    }
+
+		    break;
+		}
+
+		case Reference.Mode.ModeDatagram:
+		case Reference.Mode.ModeBatchDatagram:
+		{
+		    //
+		    // Filter out non-datagram endpoints.
+		    //
+		    for(int i = 0; i < allConnections.Length; ++i)
+		    {
+			if(allConnections[i].endpoint().datagram())
+			{
+			    connections.Add(allConnections[i]);
+			}
+		    }
+
+		    break;
+		}
+	    }
+
+	    //
+	    // Randomize the order of connections.
+	    //
+	    for(int i = 0; i < connections.Count - 2; ++i)
+	    {
+		int r = rand_.Next(connections.Count - i) + i;
+		Debug.Assert(r >= i && r < connections.Count);
+		if(r != i)
+		{
+		    object tmp = connections[i];
+		    connections[i] = connections[r];
+		    connections[r] = tmp;
+		}
+	    }
+
+	    //
+	    // If a secure connection is requested, remove all non-secure
+	    // endpoints. Otherwise make non-secure endpoints preferred over
+	    // secure endpoints by partitioning the endpoint vector, so that
+	    // non-secure endpoints come first.
+	    //
+	    if(getSecure())
+	    {
+		ArrayList tmp = new ArrayList();
+		foreach(Ice.ConnectionI connection in connections)
+		{
+		    if(connection.endpoint().secure())
+		    {
+			tmp.Add(connection);
+		    }
+		}
+		connections = tmp;
+	    }
+	    else
+	    {
+		IceUtil.Arrays.Sort(ref connections, _connectionComparator);
+	    }
+	    
+	    Ice.ConnectionI[] arr = new Ice.ConnectionI[connections.Count];
+	    if(arr.Length != 0)
+	    {
+		connections.CopyTo(arr);
+	    }
+	    return arr;
+	}
+
+	private class ConnectionComparator : IComparer
+	{
+	    public int Compare(object l, object r)
+	    {
+		Ice.ConnectionI lc = (Ice.ConnectionI)l;
+		Ice.ConnectionI rc = (Ice.ConnectionI)r;
+		bool ls = lc.endpoint().secure();
+		bool rs = rc.endpoint().secure();
+		if((ls && rs) || (!ls && !rs))
+		{
+		    return 0;
+		}
+		else if(!ls && rs)
+		{
+		    return -1;
+		}
+		else
+		{
+		    return 1;
+		}
+	    }
+	}
+	
+	private static ConnectionComparator _connectionComparator = new ConnectionComparator();
 	private Ice.ConnectionI[] _fixedConnections;
     }
 
     public abstract class RoutableReference : Reference
     {
-	public RouterInfo getRouterInfo()
+	public override RouterInfo getRouterInfo()
 	{
 	    return _routerInfo;
 	}
@@ -792,6 +717,16 @@ namespace IceInternal
 	    return _collocationOptimization;
 	}
 
+	public override bool getCacheConnection()
+	{
+	    return _cacheConnection;
+	}
+
+	public override Ice.EndpointSelectionType getEndpointSelection()
+	{
+	    return _endpointSelection;
+	}
+
 	public override Reference changeSecure(bool newSecure)
 	{
 	    if(newSecure == _secure)
@@ -806,8 +741,7 @@ namespace IceInternal
 	public override Reference changeRouter(Ice.RouterPrx newRouter)
 	{
 	    RouterInfo newRouterInfo = getInstance().routerManager().get(newRouter);
-	    if(object.ReferenceEquals(newRouterInfo, _routerInfo) ||
-		    (newRouterInfo != null && _routerInfo != null && newRouterInfo.Equals(_routerInfo)))
+	    if(newRouterInfo != null && _routerInfo != null && newRouterInfo.Equals(_routerInfo))
 	    {
 		return this;
 	    }
@@ -827,6 +761,65 @@ namespace IceInternal
 	    return r;
 	}
 
+	public override Reference changeCacheConnection(bool newCache)
+	{
+	    if(newCache == _cacheConnection)
+	    {
+		return this;
+	    }
+	    RoutableReference r = (RoutableReference)getInstance().referenceFactory().copy(this);
+	    r._cacheConnection = newCache;
+	    return r;
+	}
+
+	public override Reference changeEndpointSelection(Ice.EndpointSelectionType newType)
+	{
+	    if(newType == _endpointSelection)
+	    {
+		return this;
+	    }
+	    RoutableReference r = (RoutableReference)getInstance().referenceFactory().copy(this);
+	    r._endpointSelection = newType;
+	    return r;
+	}	
+
+	public override Reference changeCompress(bool newCompress)
+	{
+	    if(overrideCompress_ && compress_ == newCompress)
+	    {
+		return this;
+	    }
+
+	    RoutableReference r = (RoutableReference)getInstance().referenceFactory().copy(this);
+	    r.compress_ = newCompress;
+	    r.overrideCompress_ = true;
+	    return r;
+	}
+
+	public override Reference changeTimeout(int newTimeout)
+	{
+	    if(overrideTimeout_ && timeout_ == newTimeout)
+	    {
+		return this;
+	    }
+
+	    RoutableReference r = (RoutableReference)getInstance().referenceFactory().copy(this);
+	    r.timeout_ = newTimeout;
+	    r.overrideTimeout_ = true;
+	    return r;
+	}
+
+	public override Reference changeConnectionId(string id)
+	{
+	    if(connectionId_.Equals(id))
+	    {
+		return this;
+	    }
+	    RoutableReference r = (RoutableReference)getInstance().referenceFactory().copy(this);
+	    r.connectionId_ = id;
+	    return r;
+	}
+
 	public override bool Equals(object obj)
 	{
 	    //
@@ -846,6 +839,34 @@ namespace IceInternal
 	    {
 		return false;
 	    }
+	    if(_cacheConnection != rhs._cacheConnection)
+	    {
+		return false;
+	    }
+	    if(_endpointSelection != rhs._endpointSelection)
+	    {
+		return false;
+	    }
+	    if(overrideCompress_ != rhs.overrideCompress_)
+	    {
+		return false;
+	    }
+	    if(overrideCompress_ && compress_ != rhs.compress_)
+	    {
+		return false;
+	    }
+	    if(overrideTimeout_ != rhs.overrideTimeout_)
+	    {
+		return false;
+	    }
+	    if(overrideTimeout_ && timeout_ != rhs.timeout_)
+	    {
+		return false;
+	    }
+	    if(!connectionId_.Equals(rhs.connectionId_))
+	    {
+		return false;
+	    }
 	    return _routerInfo == null ? rhs._routerInfo == null : _routerInfo.Equals(rhs._routerInfo);
 	}
 
@@ -871,11 +892,230 @@ namespace IceInternal
 	    _secure = sec;
 	    _routerInfo = rtrInfo;
 	    _collocationOptimization = collocationOpt;
+	    _cacheConnection = true;
+	    _endpointSelection = Ice.EndpointSelectionType.Random;
+	    overrideCompress_ = false;
+	    compress_ = false;
+	    overrideTimeout_ = false;
+	    timeout_ = -1;
 	}
 
+
+	protected void applyOverrides(ref EndpointI[] endpts)
+        {
+	    for(int i = 0; i < endpts.Length; ++i)
+	    {
+		endpts[i] = endpts[i].connectionId(connectionId_);
+		if(overrideCompress_)
+		{
+		    endpts[i] = endpts[i].compress(compress_);
+		}
+		if(overrideTimeout_)
+		{
+		    endpts[i] = endpts[i].timeout(timeout_);
+		}
+	    }
+	}
+
+	protected Ice.ConnectionI createConnection(EndpointI[] allEndpoints, out bool comp)
+	{
+	    ArrayList endpoints = new ArrayList();
+
+	    //
+	    // Filter out unknown endpoints.
+	    //
+	    for(int i = 0; i < allEndpoints.Length; i++)
+	    {
+		if(!allEndpoints[i].unknown())
+		{
+		    endpoints.Add(allEndpoints[i]);
+		}
+	    }
+	    
+	    //
+	    // Filter out endpoints according to the mode of the reference.
+	    //
+	    switch(getMode())
+	    {
+		case Reference.Mode.ModeTwoway:
+		case Reference.Mode.ModeOneway:
+		case Reference.Mode.ModeBatchOneway:
+		{
+		    //
+		    // Filter out datagram endpoints.
+		    //
+		    ArrayList tmp = new ArrayList();
+		    foreach(EndpointI endpoint in endpoints)
+		    {
+		    	if(!endpoint.datagram())
+			{
+			    tmp.Add(endpoint);
+			}
+		    }
+		    endpoints = tmp;
+		    break;
+		}
+
+		case Reference.Mode.ModeDatagram:
+		case Reference.Mode.ModeBatchDatagram:
+		{
+		    //
+		    // Filter out non-datagram endpoints.
+		    //
+		    ArrayList tmp = new ArrayList();
+		    foreach(EndpointI endpoint in endpoints)
+		    {
+		    	if(endpoint.datagram())
+			{
+			    tmp.Add(endpoint);
+			}
+		    }
+		    endpoints = tmp;
+		    break;
+		}
+	    }
+
+	    //
+	    // Sort the endpoints according to the endpoint selection type.
+	    //
+	    switch(getEndpointSelection())
+	    {
+	        case Ice.EndpointSelectionType.Random:
+		{
+		    for(int i = 0; i < endpoints.Count - 2; ++i)
+		    {
+		        int r = rand_.Next(endpoints.Count - i) + i;
+		        Debug.Assert(r >= i && r < endpoints.Count);
+		        if(r != i)
+		        {
+			    object tmp = endpoints[i];
+			    endpoints[i] = endpoints[r];
+			    endpoints[r] = tmp;
+		        }
+		    }
+		    break;
+		}
+	        case Ice.EndpointSelectionType.Ordered:
+		{
+		    // Nothing to do.
+		    break;
+		}
+	        default:
+		{
+		    Debug.Assert(false);
+		    break;
+		}
+	    }
+
+	    //
+	    // If a secure connection is requested, remove all non-secure
+	    // endpoints. Otherwise make non-secure endpoints preferred over
+	    // secure endpoints by partitioning the endpoint vector, so that
+	    // non-secure endpoints come first.
+	    //
+	    if(getSecure())
+	    {
+		ArrayList tmp = new ArrayList();
+		foreach(EndpointI endpoint in endpoints)
+		{
+		    if(endpoint.secure())
+		    {
+			tmp.Add(endpoint);
+		    }
+		}
+		endpoints = tmp;
+	    }
+	    else
+	    {
+		IceUtil.Arrays.Sort(ref endpoints, _endpointComparator);
+	    }
+	    
+	    if(endpoints.Count == 0)
+	    {
+		Ice.NoEndpointException ex = new Ice.NoEndpointException();
+		ex.proxy = ToString();
+		throw ex;
+	    }
+	    
+	    //
+	    // Finally, create the connection.
+	    //
+	    OutgoingConnectionFactory factory = getInstance().outgoingConnectionFactory();
+	    if(getCacheConnection() || endpoints.Count == 1)
+	    {
+		//
+		// Get an existing connection or create one if there's no
+		// existing connection to one of the given endpoints.
+		//
+		EndpointI[] arr = new EndpointI[endpoints.Count];
+		endpoints.CopyTo(arr);
+		return factory.create(arr, false, out comp);
+	    }
+	    else
+	    {
+		//
+		// Go through the list of endpoints and try to create the
+		// connection until it succeeds. This is different from just
+		// calling create() with the given endpoints since this might
+		// create a new connection even if there's an existing
+		// connection for one of the endpoints.
+		//
+		Ice.LocalException exception = null;
+		EndpointI[] endpoint = new EndpointI[1];
+
+		foreach(EndpointI e in endpoints)
+		{
+		    try
+		    {
+			endpoint[0] = e;
+			return factory.create(endpoint, e != endpoints[endpoints.Count - 1], out comp);
+		    }
+		    catch(Ice.LocalException ex)
+		    {
+			exception = ex;
+		    }
+		}
+
+		Debug.Assert(exception != null);
+		throw exception;
+	    }
+	}
+
+	private class EndpointComparator : IComparer
+	{
+	    public int Compare(object l, object r)
+	    {
+		IceInternal.EndpointI le = (IceInternal.EndpointI)l;
+		IceInternal.EndpointI re = (IceInternal.EndpointI)r;
+		bool ls = le.secure();
+		bool rs = re.secure();
+		if((ls && rs) || (!ls && !rs))
+		{
+		    return 0;
+		}
+		else if(!ls && rs)
+		{
+		    return -1;
+		}
+		else
+		{
+		    return 1;
+		}
+	    }
+	}
+	
+	private static EndpointComparator _endpointComparator = new EndpointComparator();
+
 	private bool _secure;
 	private RouterInfo _routerInfo; // Null if no router is used.
 	private bool _collocationOptimization;
+	private bool _cacheConnection;
+	private Ice.EndpointSelectionType _endpointSelection;
+	private string connectionId_ = "";
+	private bool overrideCompress_;
+	private bool compress_;
+	private bool overrideTimeout_;
+	private int timeout_;
     }
 
     public class DirectReference : RoutableReference
@@ -905,54 +1145,58 @@ namespace IceInternal
 	    return _endpoints;
 	}
 
+	public override int getLocatorCacheTimeout()
+	{
+	    return 0;
+	}
+
 	public override Reference changeLocator(Ice.LocatorPrx newLocator)
 	{
-	    if(newLocator != null)
-	    {
-		LocatorInfo newLocatorInfo = getInstance().locatorManager().get(newLocator);
-		return getInstance().referenceFactory().create(
-		    getIdentity(), getContext(), getFacet(), getMode(), getSecure(), "", null, newLocatorInfo,
-		    getCollocationOptimization());
-	    }
-	    else
-	    {
-		return this;
-	    }
+	    return this;
 	}
 
 	public override Reference changeCompress(bool newCompress)
 	{
-	    DirectReference r = (DirectReference)getInstance().referenceFactory().copy(this);
-	    EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
-	    for(int i = 0; i < _endpoints.Length; i++)
+	    DirectReference r = (DirectReference)base.changeCompress(newCompress);
+	    if(r != this) // Also override the compress flag on the endpoints if it was updated.
 	    {
-		newEndpoints[i] = _endpoints[i].compress(newCompress);
+		EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
+		for(int i = 0; i < _endpoints.Length; i++)
+		{
+		    newEndpoints[i] = _endpoints[i].compress(newCompress);
+		}
+		r._endpoints = newEndpoints;
 	    }
-	    r._endpoints = newEndpoints;
 	    return r;
 	}
 
 	public override Reference changeTimeout(int newTimeout)
 	{
-	    DirectReference r = (DirectReference)getInstance().referenceFactory().copy(this);
-	    EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
-	    for(int i = 0; i < _endpoints.Length; i++)
+	    DirectReference r = (DirectReference)base.changeTimeout(newTimeout);
+	    if(r != this) // Also override the timeout on the endpoints if it was updated.
 	    {
-		newEndpoints[i] = _endpoints[i].timeout(newTimeout);
+		EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
+		for(int i = 0; i < _endpoints.Length; i++)
+		{
+		    newEndpoints[i] = _endpoints[i].timeout(newTimeout);
+		}
+		r._endpoints = newEndpoints;
 	    }
-	    r._endpoints = newEndpoints;
 	    return r;
 	}
 
 	public override Reference changeConnectionId(string connectionId)
 	{
-	    DirectReference r = (DirectReference)getInstance().referenceFactory().copy(this);
-	    EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
-	    for(int i = 0; i < _endpoints.Length; i++)
+	    DirectReference r = (DirectReference)base.changeConnectionId(connectionId);
+	    if(r != this) // Also override the connection id on the endpoints if it was updated.
 	    {
-		newEndpoints[i] = _endpoints[i].connectionId(connectionId);
+		EndpointI[] newEndpoints = new EndpointI[_endpoints.Length];
+		for(int i = 0; i < _endpoints.Length; i++)
+		{
+		    newEndpoints[i] = _endpoints[i].connectionId(connectionId);
+		}
+		r._endpoints = newEndpoints;
 	    }
-	    r._endpoints = newEndpoints;
 	    return r;
 	}
 
@@ -966,7 +1210,7 @@ namespace IceInternal
 		getInstance().locatorManager().get(getInstance().referenceFactory().getDefaultLocator());
 	    return getInstance().referenceFactory().create(
 		getIdentity(), getContext(), getFacet(), getMode(), getSecure(), newAdapterId, getRouterInfo(),
-		locatorInfo, getCollocationOptimization());
+		locatorInfo, getCollocationOptimization(), getLocatorCacheTimeout());
 	}
 
 	public override Reference changeEndpoints(EndpointI[] newEndpoints)
@@ -977,9 +1221,15 @@ namespace IceInternal
 	    }
 	    DirectReference r = (DirectReference)getInstance().referenceFactory().copy(this);
 	    r._endpoints = newEndpoints;
+	    r.applyOverrides(ref r._endpoints);
 	    return r;
 	}
 
+	public override Reference changeLocatorCacheTimeout(int newTimeout)
+        {
+	    return this;
+	}
+
 	public override void streamWrite(BasicStream s)
 	{
 	    base.streamWrite(s);
@@ -1000,6 +1250,13 @@ namespace IceInternal
 
 	public override string ToString()
 	{
+	    //
+	    // WARNING: Certain features, such as proxy validation in Glacier2,
+	    // depend on the format of proxy strings. Changes to toString() and
+	    // methods called to generate parts of the reference string could break
+	    // these features. Please review for all features that depend on the
+	    // format of proxyToString() before changing this and related code.
+	    //
 	    StringBuilder s = new StringBuilder();
 	    s.Append(base.ToString());
 
@@ -1018,20 +1275,14 @@ namespace IceInternal
 	public override Ice.ConnectionI getConnection(out bool comp)
 	{
 	    EndpointI[] endpts = base.getRoutedEndpoints();
+	    applyOverrides(ref endpts);
+
 	    if(endpts.Length == 0)
 	    {
-		endpts =_endpoints;
-	    }
-	    EndpointI[] filteredEndpoints = filterEndpoints(endpts);
-	    if(filteredEndpoints.Length == 0)
-	    {
-		Ice.NoEndpointException ex = new Ice.NoEndpointException();
-		ex.proxy = ToString();
-		throw ex;
+		endpts = _endpoints; // Endpoint overrides are already applied on these endpoints.
 	    }
 
-	    OutgoingConnectionFactory factory = getInstance().outgoingConnectionFactory();
-	    Ice.ConnectionI connection = factory.create(filteredEndpoints, out comp);
+	    Ice.ConnectionI connection = createConnection(endpts, out comp);
 
 	    //
 	    // If we have a router, set the object adapter for this router
@@ -1088,14 +1339,16 @@ namespace IceInternal
 			         string adptid,
 			         RouterInfo rtrInfo,
 			         LocatorInfo locInfo,
-			         bool collocationOpt)
+			         bool collocationOpt,
+				 int locatorCacheTimeout)
 	    : base(inst, com, ident, ctx, fs, md, sec, rtrInfo, collocationOpt)
 	{
 	    adapterId_ = adptid;
 	    locatorInfo_ = locInfo;
+	    locatorCacheTimeout_ = locatorCacheTimeout;
 	}
 
-	public LocatorInfo getLocatorInfo()
+	public override LocatorInfo getLocatorInfo()
 	{
 	    return locatorInfo_;
 	}
@@ -1110,64 +1363,20 @@ namespace IceInternal
 	    return new EndpointI[0];
 	}
 
+	public override int getLocatorCacheTimeout()
+	{
+	    return locatorCacheTimeout_;
+	}
+
 	public override Reference changeLocator(Ice.LocatorPrx newLocator)
 	{
-	    //
-	    // Return a direct reference if a null locator is given.
-	    //
-	    if(newLocator == null)
-	    {
-		return getInstance().referenceFactory().create(
-		    getIdentity(), getContext(), getFacet(), getMode(), getSecure(), new EndpointI[0], getRouterInfo(),
-		    getCollocationOptimization());
-	    }
-	    else
-	    {
-		LocatorInfo newLocatorInfo = getInstance().locatorManager().get(newLocator);
-
-		if(object.ReferenceEquals(newLocatorInfo, locatorInfo_) ||
-		    (locatorInfo_ != null && newLocatorInfo != null && newLocatorInfo.Equals(locatorInfo_)))
-		{
-		    return this;
-		}
-		IndirectReference r = (IndirectReference)getInstance().referenceFactory().copy(this);
-		r.locatorInfo_ = newLocatorInfo;
-		return this;
-	    }
-	}
-
-	public override Reference changeCompress(bool newCompress)
-	{
-	    IndirectReference r = (IndirectReference)getInstance().referenceFactory().copy(this);
-	    if(locatorInfo_ != null)
-	    {
-		Ice.LocatorPrx newLocator = Ice.LocatorPrxHelper.uncheckedCast(
-						    locatorInfo_.getLocator().ice_compress(newCompress));
-		r.locatorInfo_ = getInstance().locatorManager().get(newLocator);
-	    }
-	    return r;
-	}
-
-	public override Reference changeTimeout(int newTimeout)
-	{
-	    IndirectReference r = (IndirectReference)getInstance().referenceFactory().copy(this);
-	    if(locatorInfo_ != null)
-	    {
-		Ice.LocatorPrx newLocator = Ice.LocatorPrxHelper.uncheckedCast(
-						    locatorInfo_.getLocator().ice_timeout(newTimeout));
-		r.locatorInfo_ = getInstance().locatorManager().get(newLocator);
-	    }
-	    return r;
-	}
-
-	public override Reference changeConnectionId(string id)
-	{
-	    if(connectionId_.Equals(id))
+	    LocatorInfo newLocatorInfo = getInstance().locatorManager().get(newLocator);
+	    if(locatorInfo_ != null && newLocatorInfo != null && newLocatorInfo.Equals(locatorInfo_))
 	    {
 		return this;
 	    }
 	    IndirectReference r = (IndirectReference)getInstance().referenceFactory().copy(this);
-	    r.connectionId_ = id;
+	    r.locatorInfo_ = newLocatorInfo;
 	    return r;
 	}
 
@@ -1193,6 +1402,17 @@ namespace IceInternal
 		getCollocationOptimization());
 	}
 
+	public override Reference changeLocatorCacheTimeout(int newTimeout)
+        {
+	    if(newTimeout == locatorCacheTimeout_)
+	    {
+		return this;
+	    }
+	    IndirectReference r = (IndirectReference)getInstance().referenceFactory().copy(this);
+	    r.locatorCacheTimeout_ = newTimeout;
+	    return r;
+	}
+
 	public override void streamWrite(BasicStream s)
 	{
 	    base.streamWrite(s);
@@ -1203,6 +1423,13 @@ namespace IceInternal
 
 	public override string ToString()
 	{
+	    //
+	    // WARNING: Certain features, such as proxy validation in Glacier2,
+	    // depend on the format of proxy strings. Changes to toString() and
+	    // methods called to generate parts of the reference string could break
+	    // these features. Please review for all features that depend on the
+	    // format of proxyToString() before changing this and related code.
+	    //
 	    string result = base.ToString();
 
 	    if(adapterId_.Length == 0)
@@ -1243,29 +1470,20 @@ namespace IceInternal
 		bool cached = false;
 		if(endpts.Length == 0 && locatorInfo_ != null)
 		{
-		    endpts = locatorInfo_.getEndpoints(this, out cached);
-		}
-		//
-		// Apply the cached connection id to each endpoint.
-		//
-		for(int i = 0; i < endpts.Length; ++i)
-		{
-		    endpts[i] = endpts[i].connectionId(connectionId_);
-		}
-		EndpointI[] filteredEndpoints = filterEndpoints(endpts);
-		if(filteredEndpoints.Length == 0)
-		{
-		    Ice.NoEndpointException ex = new Ice.NoEndpointException();
-		    ex.proxy = ToString();
-		    throw ex;
+		    endpts = locatorInfo_.getEndpoints(this, locatorCacheTimeout_, out cached);
 		}
 
+		applyOverrides(ref endpts);
+
 		try
 		{
-		    OutgoingConnectionFactory factory = getInstance().outgoingConnectionFactory();
-		    connection = factory.create(filteredEndpoints, out comp);
+		    connection = createConnection(endpts, out comp);
 		    Debug.Assert(connection != null);
 		}
+		catch(Ice.NoEndpointException ex)
+		{
+		    throw ex; // No need to retry if there's no endpoints.
+		}
 		catch(Ice.LocalException ex)
 		{
 		    if(getRouterInfo() == null)
@@ -1276,13 +1494,12 @@ namespace IceInternal
 			if(cached)
 			{
 			    TraceLevels traceLevels = getInstance().traceLevels();
-			    Ice.Logger logger = getInstance().logger();
 
 			    if(traceLevels.retry >= 2)
 			    {
 				String s = "connection to cached endpoints failed\n" +
 				           "removing endpoints from cache and trying one more time\n" + ex;
-				logger.trace(traceLevels.retryCat, s);
+				getInstance().initializationData().logger.trace(traceLevels.retryCat, s);
 			    }
 			    
 			    continue;
@@ -1328,12 +1545,11 @@ namespace IceInternal
 	    {
 		return false;
 	    }
-	    if(!connectionId_.Equals(rhs.connectionId_))
+	    if(locatorInfo_ == null ? rhs.locatorInfo_ != null : !locatorInfo_.Equals(rhs.locatorInfo_))
 	    {
 		return false;
 	    }
-	    return object.ReferenceEquals(locatorInfo_, rhs.locatorInfo_) ||
-		(locatorInfo_ != null && rhs.locatorInfo_ != null && rhs.locatorInfo_.Equals(locatorInfo_));
+	    return locatorCacheTimeout_ == rhs.locatorCacheTimeout_;
 	}
 
         //
@@ -1341,11 +1557,24 @@ namespace IceInternal
         //
 	public override int GetHashCode()
 	{
-            return base.GetHashCode();
+	    lock(this)
+	    {
+		if(base.hashInitialized_)
+		{
+		    return hashValue_;
+		}
+		base.GetHashCode();         // Initializes hashValue_.
+		int sz = adapterId_.Length; // Add hash of adapter ID to base hash.
+		for(int i = 0; i < sz; i++)
+		{   
+		    hashValue_ = 5 * hashValue_ + (int)adapterId_[i];
+		}
+		return hashValue_;
+	    }
         }
 
 	private string adapterId_;
-	private string connectionId_ = "";
 	private LocatorInfo locatorInfo_;
+	private int locatorCacheTimeout_;
     }
 }
