@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -45,7 +45,7 @@ namespace IceInternal
 	    inUse_ = 0;
 	    load_ = 1.0;
 	    promote_ = true;
-	    warnUdp_ = instance_.properties().getPropertyAsInt("Ice.Warn.Datagrams") > 0;
+	    warnUdp_ = instance_.initializationData().properties.getPropertyAsInt("Ice.Warn.Datagrams") > 0;
 
 	    //
 	    // If we are in thread per connection mode, no thread pool should
@@ -53,7 +53,7 @@ namespace IceInternal
 	    //
 	    Debug.Assert(!instance_.threadPerConnection());
 
-	    string programName = instance_.properties().getProperty("Ice.ProgramName");
+	    string programName = instance_.initializationData().properties.getProperty("Ice.ProgramName");
 	    if(programName.Length > 0)
 	    {
 		_programNamePrefix = programName + "-";
@@ -73,20 +73,33 @@ namespace IceInternal
 	    // possible setting, still allows one level of nesting, and
 	    // doesn't require to make the servants thread safe.
 	    //
-	    int size = instance_.properties().getPropertyAsIntWithDefault(_prefix + ".Size", 1);
+	    int size = instance_.initializationData().properties.getPropertyAsIntWithDefault(_prefix + ".Size", 1);
 	    if(size < 1)
 	    {
+		string s = _prefix + ".Size < 1; Size adjusted to 1";
+		instance_.initializationData().logger.warning(s);
 		size = 1;
 	    }
 	    
-	    int sizeMax = instance_.properties().getPropertyAsIntWithDefault(_prefix + ".SizeMax", size);
+	    int sizeMax = 
+	        instance_.initializationData().properties.getPropertyAsIntWithDefault(_prefix + ".SizeMax", size);
 	    if(sizeMax < size)
 	    {
+		string s = _prefix + ".SizeMax < " + _prefix + ".Size; SizeMax adjusted to Size (" + size + ")";
+		instance_.initializationData().logger.warning(s);
 		sizeMax = size;
 	    }
 	    
-	    int sizeWarn = instance_.properties().getPropertyAsIntWithDefault(_prefix + ".SizeWarn",
+	    int sizeWarn = instance_.initializationData().properties.getPropertyAsIntWithDefault(_prefix + ".SizeWarn",
 	                                                                      sizeMax * 80 / 100);
+	    if(sizeWarn > sizeMax)
+	    {
+		string s = _prefix + ".SizeWarn > " + _prefix + ".SizeMax; adjusted SizeWarn to SizeMax ("
+		    + sizeMax + ")";
+		instance_.initializationData().logger.warning(s);
+		sizeWarn = sizeMax;
+	    }
+
 	    size_ = size;
 	    sizeMax_ = sizeMax;
 	    sizeWarn_ = sizeWarn;
@@ -106,7 +119,7 @@ namespace IceInternal
 	    catch(System.Exception ex)
 	    {
 		string s = "cannot create thread for `" + _prefix + "':\n" + ex;
-		instance_.logger().error(s);
+		instance_.initializationData().logger.error(s);
 		
 		destroy();
 		joinWithAllThreads();
@@ -114,15 +127,15 @@ namespace IceInternal
 	    }
 	}
 	
+#if DEBUG
 	~ThreadPool()
 	{
-#if DEBUG
 	    lock(this)
 	    {
 		IceUtil.Assert.FinalizerAssert(_destroyed);
 	    }
-#endif
 	}
+#endif
 	
 	public void destroy()
 	{
@@ -198,7 +211,7 @@ namespace IceInternal
 			    string s = "thread pool `" + _prefix + "' is running low on threads\n"
 				       + "Size=" + size_ + ", " + "SizeMax=" + sizeMax_ + ", "
 				       + "SizeWarn=" + sizeWarn_;
-			    instance_.logger().warning(s);
+			    instance_.initializationData().logger.warning(s);
 			}
 			
 			Debug.Assert(inUse_ <= running_);
@@ -215,7 +228,7 @@ namespace IceInternal
 			    catch(System.Exception ex)
 			    {
 				string s = "cannot create thread for `" + _prefix + "':\n" + ex;
-				instance_.logger().error(s);
+				instance_.initializationData().logger.error(s);
 			    }
 			}
 		    }
@@ -531,7 +544,7 @@ namespace IceInternal
 			{
 			    string s = "exception in `" + _prefix + "' while calling finished():\n"
 			               + ex + "\n" + handler.ToString();
-			    instance_.logger().error(s);
+			    instance_.initializationData().logger.error(s);
 			}
 
 			//
@@ -577,9 +590,11 @@ namespace IceInternal
 				{
 				    if(handler.datagram())
 				    {
-				        if(instance_.properties().getPropertyAsInt("Ice.Warn.Connections") > 0)
+				        if(instance_.initializationData().properties.getPropertyAsInt(
+											"Ice.Warn.Connections") > 0)
 					{
-					    instance_.logger().warning("datagram connection exception:\n" + ex + 
+					    instance_.initializationData().logger.warning(
+					    			       "datagram connection exception:\n" + ex + 
 					    			       handler.ToString());
 					}
 				    }
@@ -609,7 +624,7 @@ namespace IceInternal
 			    catch(Ice.LocalException ex)
 			    {
 			        string s = "exception in `" + _prefix + "' while calling message():\n" + ex;
-			    	instance_.logger().error(s);
+			    	instance_.initializationData().logger.error(s);
 			    }
 
 			    //
@@ -737,7 +752,13 @@ namespace IceInternal
 	    }
 	    
 	    int pos = stream.pos();
-	    Debug.Assert(pos >= Protocol.headerSize);
+	    if(pos < Protocol.headerSize)
+	    {
+		//
+		// This situation is possible for small UDP packets.
+		//
+		throw new Ice.IllegalMessageSizeException();
+	    }
 	    stream.pos(0);
 	    byte[] m = new byte[4];
 	    m[0] = stream.readByte();
@@ -799,8 +820,8 @@ namespace IceInternal
 		{
 		    if(warnUdp_)
 		    {
-			instance_.logger().warning("DatagramLimitException: maximum size of " + stream.pos() +
-						   " exceeded");
+			instance_.initializationData().logger.warning("DatagramLimitException: maximum size of " +
+								      stream.pos() + " exceeded");
 		    }
 		    stream.pos(0);
 		    stream.resize(0, true);
@@ -912,6 +933,11 @@ namespace IceInternal
 
 	    public void Run()
 	    {
+	        if(thread_Pool.instance_.initializationData().threadHook != null)
+		{
+		    thread_Pool.instance_.initializationData().threadHook.start();
+		}
+
 		BasicStream stream = new BasicStream(thread_Pool.instance_);
 		
 		bool promote;
@@ -923,13 +949,13 @@ namespace IceInternal
 		catch(Ice.LocalException ex)
 		{
 		    string s = "exception in `" + thread_Pool._prefix + "' thread " + thread_.Name + ":\n" + ex;
-		    thread_Pool.instance_.logger().error(s);
+		    thread_Pool.instance_.initializationData().logger.error(s);
 		    promote = true;
 		}
 		catch(System.Exception ex)
 		{
 		    string s = "unknown exception in `" + thread_Pool._prefix + "' thread " + thread_.Name + ":\n" + ex;
-		    thread_Pool.instance_.logger().error(s);
+		    thread_Pool.instance_.initializationData().logger.error(s);
 		    promote = true;
 		}
 		
@@ -946,6 +972,11 @@ namespace IceInternal
 			System.Threading.Monitor.Pulse(thread_Pool);
 		    }
 		}
+
+	        if(thread_Pool.instance_.initializationData().threadHook != null)
+		{
+		    thread_Pool.instance_.initializationData().threadHook.stop();
+		}
 		
 		#if TRACE_THREAD
 		    thread_Pool.trace("run() terminated");
