@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,7 +17,7 @@ namespace IceInternal
     {
 	internal const short TYPE = 3;
 	
-	public UdpEndpointI(Instance instance, string ho, int po, string conId, bool co, bool pub)
+	public UdpEndpointI(Instance instance, string ho, int po, bool conn, string conId, bool co, bool pub)
 	{
 	    instance_ = instance;
 	    _host = ho;
@@ -26,14 +26,14 @@ namespace IceInternal
 	    _protocolMinor = Protocol.protocolMinor;
 	    _encodingMajor = Protocol.encodingMajor;
 	    _encodingMinor = Protocol.encodingMinor;
-	    _connect = false;
+	    _connect = conn;
 	    _connectionId = conId;
 	    _compress = co;
 	    _publish = pub;
 	    calcHashValue();
 	}
 	
-	public UdpEndpointI(Instance instance, string str, bool adapterEndp)
+	public UdpEndpointI(Instance instance, string str)
 	{
 	    instance_ = instance;
 	    _host = null;
@@ -269,17 +269,10 @@ namespace IceInternal
 		_host = instance.defaultsAndOverrides().defaultHost;
 		if(_host == null)
 		{
-                    if(adapterEndp)
-                    {
-                        _host = "0.0.0.0";
-                    }
-                    else
-                    {
-                        _host = Network.getLocalHost(true);
-                    }
+                    _host = "0.0.0.0";
 		}
 	    }
-	    else if(_host.Equals("*") && adapterEndp)
+	    else if(_host.Equals("*"))
 	    {
 	        _host = "0.0.0.0";
 	    }
@@ -348,18 +341,25 @@ namespace IceInternal
 	//
 	public override string ice_toString_()
 	{
+	    //
+	    // WARNING: Certain features, such as proxy validation in Glacier2,
+	    // depend on the format of proxy strings. Changes to toString() and
+	    // methods called to generate parts of the reference string could break
+	    // these features. Please review for all features that depend on the
+	    // format of proxyToString() before changing this and related code.
+	    //
 	    string s = "udp";
 
 	    if((int)_protocolMajor != 1 || (int)_protocolMinor != 0)
 	    {
-	    	s += " -v " + (_protocolMajor < 0 ? (int)_protocolMajor + 255 : _protocolMajor);
-	    	s += "." + (_protocolMinor < 0 ? (int)_protocolMinor + 255 : _protocolMinor);
+		s += " -v " + (_protocolMajor < 0 ? (int)_protocolMajor + 255 : _protocolMajor);
+		s += "." + (_protocolMinor < 0 ? (int)_protocolMinor + 255 : _protocolMinor);
 	    }
 
 	    if((int)_encodingMajor != 1 || (int)_encodingMinor != 0)
 	    {
 		s += " -e " + (_encodingMajor < 0 ? (int)_encodingMajor + 255 : _encodingMajor);
-	    	s += "." + (_encodingMinor < 0 ? (int)_encodingMinor + 255 : _encodingMinor);
+		s += "." + (_encodingMinor < 0 ? (int)_encodingMinor + 255 : _encodingMinor);
 	    }
 
 	    s += " -h " + _host + " -p " + _port;
@@ -416,7 +416,7 @@ namespace IceInternal
 	    }
 	    else
 	    {
-		return new UdpEndpointI(instance_, _host, _port, _connectionId, compress, _publish);
+		return new UdpEndpointI(instance_, _host, _port, _connect, _connectionId, compress, _publish);
 	    }
 	}
 
@@ -431,7 +431,7 @@ namespace IceInternal
 	    }
 	    else
 	    {
-		return new UdpEndpointI(instance_, _host, _port, connectionId, _compress, _publish);
+		return new UdpEndpointI(instance_, _host, _port, _connect, connectionId, _compress, _publish);
 	    }
 	}
 	
@@ -488,7 +488,8 @@ namespace IceInternal
 	public override Transceiver serverTransceiver(ref EndpointI endpoint)
 	{
 	    UdpTransceiver p = new UdpTransceiver(instance_, _host, _port, _connect);
-	    endpoint = new UdpEndpointI(instance_, _host, p.effectivePort(), _connectionId, _compress, _publish);
+	    endpoint = new UdpEndpointI(instance_, _host, p.effectivePort(), _connect, _connectionId, _compress,
+					_publish);
 	    return p;
 	}
 	
@@ -508,7 +509,7 @@ namespace IceInternal
 	// from this endpoint, for example, if a dynamic port number is
 	// assigned.
 	//
-	public override Acceptor acceptor(ref EndpointI endpoint)
+	public override Acceptor acceptor(ref EndpointI endpoint, string adapterName)
 	{
 	    endpoint = this;
 	    return null;
@@ -520,7 +521,7 @@ namespace IceInternal
         // only applies for ObjectAdapter endpoints.
         //
         public override ArrayList
-        expand()
+        expand(bool includeLoopback)
         {
             ArrayList endps = new ArrayList();
             if(_host.Equals("0.0.0.0"))
@@ -528,8 +529,11 @@ namespace IceInternal
                 string[] hosts = Network.getLocalHosts();
                 for(int i = 0; i < hosts.Length; ++i)
                 {
-                    endps.Add(new UdpEndpointI(instance_, hosts[i], _port, _connectionId, _compress,
-                                               hosts.Length == 1 || !hosts[i].Equals("127.0.0.1")));
+		    if(includeLoopback || hosts.Length == 1 || !hosts[i].Equals("127.0.0.1"))
+		    {
+                        endps.Add(new UdpEndpointI(instance_, hosts[i], _port, _connect, _connectionId, _compress,
+                                                   hosts.Length == 1 || !hosts[i].Equals("127.0.0.1")));
+		    }
                 }
             }
             else
@@ -771,9 +775,9 @@ namespace IceInternal
             return "udp";
         }
 	
-        public EndpointI create(string str, bool adapterEndp)
+        public EndpointI create(string str)
         {
-            return new UdpEndpointI(instance_, str, adapterEndp);
+            return new UdpEndpointI(instance_, str);
         }
 	
         public EndpointI read(BasicStream s)
