@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,16 +13,16 @@ public class Server : Ice.Application
 {
     public override int run(string[] args)
     {
-	communicator().getProperties().setProperty("CallbackAdapter.Endpoints", "tcp -p 12345 -t 10000");
+	communicator().getProperties().setProperty("CallbackAdapter.Endpoints", "tcp -p 12010 -t 10000");
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("CallbackAdapter");
 	adapter.add(new CallbackI(),
-		    Ice.Util.stringToIdentity("c1/callback")); // The test allows "c1" as category.
+		    communicator().stringToIdentity("c1/callback")); // The test allows "c1" as category.
 	adapter.add(new CallbackI(),
-		    Ice.Util.stringToIdentity("c2/callback")); // The test allows "c2" as category.
+		    communicator().stringToIdentity("c2/callback")); // The test allows "c2" as category.
 	adapter.add(new CallbackI(),
-		    Ice.Util.stringToIdentity("c3/callback")); // The test rejects "c3" as category.
+		    communicator().stringToIdentity("c3/callback")); // The test rejects "c3" as category.
 	adapter.add(new CallbackI(),
-		    Ice.Util.stringToIdentity("_userid/callback")); // The test allows the prefixed userid.
+		    communicator().stringToIdentity("_userid/callback")); // The test allows the prefixed userid.
         adapter.activate();
         communicator().waitForShutdown();
         return 0;
@@ -32,6 +32,9 @@ public class Server : Ice.Application
     {
         Server app = new Server();
         int status = app.main(args);
-	Environment.Exit(status);
+	if(status != 0)
+	{
+	    Environment.Exit(status);
+	}
     }
 }
