@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -24,6 +24,9 @@ public class Client
     {
         TestClient app = new TestClient();
         int result = app.main(args);
-        System.Environment.Exit(result);
+	if(result != 0)
+	{
+	    System.Environment.Exit(result);
+	}
     }
 }
