@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -15,7 +15,7 @@ public class Collocated
     {
         public override int run(string[] args)
         {
-	    communicator().getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 10000");
+	    communicator().getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 10000");
 	    Ice.ObjectAdapter adapter = communicator().createObjectAdapter("TestAdapter");
 	    Ice.ServantLocator locator = new ServantLocatorI();
 	    adapter.addServantLocator(locator, "");
@@ -31,6 +31,9 @@ public class Collocated
     {
         TestClient app = new TestClient();
         int result = app.main(args);
-        System.Environment.Exit(result);
+	if(result != 0)
+	{
+	    System.Environment.Exit(result);
+	}
     }
 }
