@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,7 +13,7 @@ public class Server
     {
         public override int run(string[] args)
         {
-            communicator().getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 2000:udp");
+            communicator().getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 2000:udp");
             Ice.ObjectAdapter adapter = communicator().createObjectAdapter("TestAdapter");
             Ice.ServantLocator locator = new ServantLocatorI();
             adapter.addServantLocator(locator, "");
@@ -27,6 +27,9 @@ public class Server
     {
         TestServer app = new TestServer();
         int result = app.main(args);
-        System.Environment.Exit(result);
+	if(result != 0)
+	{
+	    System.Environment.Exit(result);
+	}
     }
 }
