@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,10 +13,10 @@ public class Server
 {
     private static int run(string[] args, Ice.Communicator communicator)
     {
-	communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 2000");
+	communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 2000");
 	Ice.ObjectAdapter adapter = communicator.createObjectAdapter("TestAdapter");
 	Ice.Object @object = new Test.ChecksumI(adapter);
-	adapter.add(@object, Ice.Util.stringToIdentity("test"));
+	adapter.add(@object, communicator.stringToIdentity("test"));
 	adapter.activate();
 	communicator.waitForShutdown();
 	return 0;
@@ -51,6 +51,9 @@ public class Server
 	    }
 	}
 	
-	Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
