@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -102,13 +102,14 @@ public class AllTests
     
     private class AMI_Thrower_throwAasAObjectNotExistI : AMI_Thrower_throwAasA
     {
-	public AMI_Thrower_throwAasAObjectNotExistI()
+	public AMI_Thrower_throwAasAObjectNotExistI(Ice.Communicator comm)
 	{
-	    InitBlock();
+	    InitBlock(comm);
 	}
-	private void InitBlock()
+	private void InitBlock(Ice.Communicator comm)
 	{
 	    callback = new Callback();
+	    communicator = comm;
 	}
 	public override void ice_response()
 	{
@@ -123,7 +124,7 @@ public class AllTests
 	    }
 	    catch(Ice.ObjectNotExistException ex)
 	    {
-		Ice.Identity id = Ice.Util.stringToIdentity("does not exist");
+		Ice.Identity id = communicator.stringToIdentity("does not exist");
 		AllTests.test(ex.id.Equals(id));
 	    }
 	    catch(Exception)
@@ -138,6 +139,7 @@ public class AllTests
 	    return callback.check();
 	}
 	
+	private Ice.Communicator communicator;
 	private Callback callback;
     }
     
@@ -679,7 +681,7 @@ public class AllTests
 	    try
 	    {
 		Ice.ObjectAdapter second = 
-		    communicator.createObjectAdapterWithEndpoints("TestAdapter0", "ssl -h foo -p 12346 -t 10000");
+		    communicator.createObjectAdapterWithEndpoints("TestAdapter0", "ssl -h foo -p 12011 -t 10000");
 		test(false);
 
 		//
@@ -700,20 +702,20 @@ public class AllTests
             Console.Write("testing servant registration exceptions... ");
             Ice.ObjectAdapter adapter = communicator.createObjectAdapter("TestAdapter1");
             Ice.Object obj = new EmptyI();
-            adapter.add(obj, Ice.Util.stringToIdentity("x"));
+            adapter.add(obj, communicator.stringToIdentity("x"));
             try
             {
-                adapter.add(obj, Ice.Util.stringToIdentity("x"));
+                adapter.add(obj, communicator.stringToIdentity("x"));
 		test(false);
             }
             catch(Ice.AlreadyRegisteredException)
             {
             }
             
-            adapter.remove(Ice.Util.stringToIdentity("x"));
+            adapter.remove(communicator.stringToIdentity("x"));
             try
             {
-                adapter.remove(Ice.Util.stringToIdentity("x"));
+                adapter.remove(communicator.stringToIdentity("x"));
 		test(false);
             }
             catch(Ice.NotRegisteredException)
@@ -742,7 +744,7 @@ public class AllTests
         }
         
         {
-            Console.Write("testing object factory registration exceptions... ");
+            Console.Write("testing object factory registration exception... ");
             Ice.ObjectFactory of = new ObjectFactoryI();
             communicator.addObjectFactory(of, "::x");
             try
@@ -753,22 +755,12 @@ public class AllTests
             catch(Ice.AlreadyRegisteredException)
             {
             }
-            
-            communicator.removeObjectFactory("::x");
-            try
-            {
-                communicator.removeObjectFactory("::x");
-		test(false);
-            }
-            catch(Ice.NotRegisteredException)
-            {
-            }
             Console.WriteLine("ok");
         }
         
         Console.Write("testing stringToProxy... ");
         Console.Out.Flush();
-        String @ref = "thrower:default -p 12345 -t 2000";
+        String @ref = "thrower:default -p 12010 -t 2000";
         Ice.ObjectPrx @base = communicator.stringToProxy(@ref);
         test(@base != null);
         Console.WriteLine("ok");
@@ -1040,10 +1032,10 @@ public class AllTests
         Console.Out.Flush();
         
         {
-            Ice.Identity id = Ice.Util.stringToIdentity("does not exist");
+            Ice.Identity id = communicator.stringToIdentity("does not exist");
             try
             {
-                ThrowerPrx thrower2 = ThrowerPrxHelper.uncheckedCast(thrower.ice_newIdentity(id));
+                ThrowerPrx thrower2 = ThrowerPrxHelper.uncheckedCast(thrower.ice_identity(id));
                 thrower2.ice_ping();
                 test(false);
             }
@@ -1250,9 +1242,9 @@ public class AllTests
             Console.Out.Flush();
             
             {
-                Ice.Identity id = Ice.Util.stringToIdentity("does not exist");
-                ThrowerPrx thrower2 = ThrowerPrxHelper.uncheckedCast(thrower.ice_newIdentity(id));
-                AMI_Thrower_throwAasAObjectNotExistI cb = new AMI_Thrower_throwAasAObjectNotExistI();
+                Ice.Identity id = communicator.stringToIdentity("does not exist");
+                ThrowerPrx thrower2 = ThrowerPrxHelper.uncheckedCast(thrower.ice_identity(id));
+                AMI_Thrower_throwAasAObjectNotExistI cb = new AMI_Thrower_throwAasAObjectNotExistI(communicator);
                 thrower2.throwAasA_async(cb, 1);
                 test(cb.check());
             }
