@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -12,10 +12,10 @@ public class Collocated
     private static int
     run(string[] args, Ice.Communicator communicator)
     {
-        communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 2000");
+        communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 2000");
         Ice.ObjectAdapter adapter = communicator.createObjectAdapter("TestAdapter");
         Ice.Object obj = new ThrowerI(adapter);
-        adapter.add(obj, Ice.Util.stringToIdentity("thrower"));
+        adapter.add(obj, communicator.stringToIdentity("thrower"));
         AllTests.allTests(communicator, true);
 	return 0;
     }
@@ -50,6 +50,9 @@ public class Collocated
 	    }
 	}
 	
-	System.Environment.Exit(status);
+	if(status != 0)
+	{
+	    System.Environment.Exit(status);
+	}
     }
 }
