@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # **********************************************************************
 #
-# Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+# Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 #
 # This copy of Ice is licensed to you under the terms described in the
 # ICE_LICENSE file included in this distribution.
@@ -20,29 +20,12 @@ else:
 sys.path.append(os.path.join(toplevel, "config"))
 import TestUtil
 
-def usage():
-    print "usage: " + sys.argv[0] + " [-m]"
-    sys.exit(2)
-
-try:
-    opts, args = getopt.getopt(sys.argv[1:], "m")
-except getopt.GetoptError:
-    usage()
-
-mono = 0
-for o, a in opts:
-    if o == "-m":
-        mono = 1
-    
-if not TestUtil.isWin32():
-    mono = 1
-
 name = os.path.join("Ice", "exceptions")
 
 print "tests with regular server."
-TestUtil.clientServerTestWithOptions(mono, name, "", " --Ice.Warn.Connections=0")
+TestUtil.clientServerTestWithOptions(name, "", " --Ice.Warn.Connections=0")
 print "tests with AMD server."
-TestUtil.clientServerTestWithOptionsAndNames(mono, name, "", "", "serveramd", "client")
+TestUtil.clientServerTestWithOptionsAndNames(name, "", "", "serveramd", "client")
 print "tests with collocated server."
-TestUtil.collocatedTest(mono, name)
+TestUtil.collocatedTest(name)
 sys.exit(0)
