@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -22,9 +22,10 @@ public class Client
         
         try
         {
-            Ice.Properties properties = Ice.Util.createProperties(ref args);
-            properties.setProperty("Ice.Default.Locator", "locator:default -p 12345");
-            communicator = Ice.Util.initializeWithProperties(ref args, properties);
+	    Ice.InitializationData initData = new Ice.InitializationData();
+            initData.properties = Ice.Util.createProperties(ref args);
+            initData.properties.setProperty("Ice.Default.Locator", "locator:default -p 12010");
+            communicator = Ice.Util.initialize(ref args, initData);
             status = run(args, communicator);
         }
         catch(System.Exception ex)
@@ -46,6 +47,9 @@ public class Client
             }
         }
         
-        System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
