@@ -1,29 +1,32 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
 //
 // **********************************************************************
 
-public class ServerLocator : Ice.LocatorDisp_
+public class ServerLocator : Test.TestLocatorDisp_
 {
     public ServerLocator(ServerLocatorRegistry registry, Ice.LocatorRegistryPrx registryPrx)
     {
         _registry = registry;
         _registryPrx = registryPrx;
+	_requestCount = 0;
     }
     
     public override void findAdapterById_async(Ice.AMD_Locator_findAdapterById response, string adapter,
     					       Ice.Current current)
     {
+	++_requestCount;
         response.ice_response(_registry.getAdapter(adapter));
     }
     
     public override void findObjectById_async(Ice.AMD_Locator_findObjectById response, Ice.Identity id,
     					      Ice.Current current)
     {
+	++_requestCount;
         response.ice_response(_registry.getObject(id));
     }
     
@@ -31,7 +34,13 @@ public class ServerLocator : Ice.LocatorDisp_
     {
         return _registryPrx;
     }
-    
+
+    public override int getRequestCount(Ice.Current current)
+    {
+	return _requestCount;
+    }
+
     private ServerLocatorRegistry _registry;
     private Ice.LocatorRegistryPrx _registryPrx;
+    private int _requestCount;
 }
