@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -17,7 +17,7 @@ public class TestI : TestIntfDisp_
         _adapter2 = adapter2;
 	_registry = registry;
         
-	_registry.addObject(_adapter1.add(new HelloI(), Ice.Util.stringToIdentity("hello")));
+	_registry.addObject(_adapter1.add(new HelloI(), adapter1.getCommunicator().stringToIdentity("hello")));
     }
     
     public override void shutdown(Ice.Current current)
@@ -27,12 +27,19 @@ public class TestI : TestIntfDisp_
     
     public override HelloPrx getHello(Ice.Current current)
     {
-        return HelloPrxHelper.uncheckedCast(_adapter1.createProxy(Ice.Util.stringToIdentity("hello")));
+        return HelloPrxHelper.uncheckedCast(_adapter1.createIndirectProxy(
+							_adapter1.getCommunicator().stringToIdentity("hello")));
+    }
+
+    public override HelloPrx getReplicatedHello(Ice.Current current)
+    {
+        return HelloPrxHelper.uncheckedCast(_adapter1.createProxy(
+							_adapter1.getCommunicator().stringToIdentity("hello")));
     }
 
     public override void migrateHello(Ice.Current current)
     {
-	Ice.Identity id = Ice.Util.stringToIdentity("hello");
+	Ice.Identity id = _adapter1.getCommunicator().stringToIdentity("hello");
 	try
 	{
 	    _registry.addObject(_adapter2.add(_adapter1.remove(id), id));
