@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -14,10 +14,10 @@ public class Client
     private static int
     run(string[] args, Ice.Communicator communicator)
     {
-        communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 2000");
+        communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 2000");
         Ice.ObjectAdapter adapter = communicator.createObjectAdapter("TestAdapter");
         Initial initial = new InitialI(adapter);
-        adapter.add(initial, Ice.Util.stringToIdentity("initial"));
+        adapter.add(initial, communicator.stringToIdentity("initial"));
         AllTests.allTests(communicator, true);
 	// We must call shutdown even in the collocated case for cyclic dependency cleanup
 	initial.shutdown();
@@ -54,6 +54,9 @@ public class Client
 	    }
 	}
 	
-	System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
