@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,11 +13,16 @@ public class Server
 {
     private static int run(string[] args, Ice.Communicator communicator)
     {
-	communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12345 -t 2000:udp");
+	//
+	// We don't want connection warnings because of the timeout test.
+	//
+	communicator.getProperties().setProperty("Ice.Warn.Connections", "0");
+	
+	communicator.getProperties().setProperty("TestAdapter.Endpoints", "default -p 12010 -t 2000:udp");
 	Ice.ObjectAdapter adapter = communicator.createObjectAdapter("TestAdapter");
-	Ice.Identity id = Ice.Util.stringToIdentity("test");
+	Ice.Identity id = communicator.stringToIdentity("test");
 	adapter.add(new MyDerivedClassI(adapter, id), id);
-	adapter.add(new TestCheckedCastI(), Ice.Util.stringToIdentity("context"));
+	adapter.add(new TestCheckedCastI(), communicator.stringToIdentity("context"));
 	adapter.activate();
 
 	communicator.waitForShutdown();
@@ -53,6 +58,9 @@ public class Server
 	    }
 	}
 	
-	Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
