@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -19,7 +19,8 @@ class Twoways
 	}
     }
     
-    internal static void twoways(Ice.Communicator communicator, Test.MyClassPrx p)
+    internal static void twoways(Ice.Communicator communicator, 
+				 Ice.InitializationData initData, Test.MyClassPrx p)
     {
 	{
 	    p.opVoid();
@@ -116,9 +117,9 @@ class Twoways
 	    test(Ice.Util.proxyIdentityAndFacetCompare(c1, p) == 0);
 	    test(Ice.Util.proxyIdentityAndFacetCompare(c2, p) != 0);
 	    test(Ice.Util.proxyIdentityAndFacetCompare(r, p) == 0);
-	    test(c1.ice_getIdentity().Equals(Ice.Util.stringToIdentity("test")));
-	    test(c2.ice_getIdentity().Equals(Ice.Util.stringToIdentity("noSuchIdentity")));
-	    test(r.ice_getIdentity().Equals(Ice.Util.stringToIdentity("test")));
+	    test(c1.ice_getIdentity().Equals(communicator.stringToIdentity("test")));
+	    test(c2.ice_getIdentity().Equals(communicator.stringToIdentity("noSuchIdentity")));
+	    test(r.ice_getIdentity().Equals(communicator.stringToIdentity("test")));
 	    r.opVoid();
 	    c1.opVoid();
 	    try
@@ -589,7 +590,7 @@ class Twoways
 		test(r.Equals(ctx));
 	    }
 	    {
-		Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_newContext(ctx));
+		Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_context(ctx));
 		test(p2.ice_getContext().Equals(ctx));
 		Ice.Context r = p2.opContext();
 		test(r.Equals(ctx));
@@ -605,7 +606,7 @@ class Twoways
 		communicator.setDefaultContext(dflt);
 		test(!p.opContext().Equals(dflt));
 
-		Test.MyClassPrx p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_newContext(new Ice.Context()));
+		Test.MyClassPrx p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_context(new Ice.Context()));
 		test(p2.opContext().Count == 0);
 
 		p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_defaultContext());
@@ -616,15 +617,15 @@ class Twoways
 
 		communicator.setDefaultContext(dflt);
 		Test.MyClassPrx c = Test.MyClassPrxHelper.checkedCast(
-					communicator.stringToProxy("test:default -p 12345 -t 10000"));
+					communicator.stringToProxy("test:default -p 12010 -t 10000"));
 		test(c.opContext().Equals(dflt));
 
 		dflt["a"] = "c";
-		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_newContext(dflt));
+		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_context(dflt));
 		test(c2.opContext()["a"].Equals("c"));
 
 		dflt.Clear();
-		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_newContext(dflt));
+		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_context(dflt));
 		test(c3.opContext()["a"] == null);
 
 		Test.MyClassPrx c4 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_defaultContext());
