@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -258,13 +258,18 @@ public class TwowaysAMI
     
     private class AMI_MyClass_opMyClassI : Test.AMI_MyClass_opMyClass
     {
+        public AMI_MyClass_opMyClassI(Ice.Communicator comunicator)
+	{
+	    _communicator = comunicator;
+	}
+
         public override void ice_response(Test.MyClassPrx r, Test.MyClassPrx c1, Test.MyClassPrx c2)
         {
-            test(c1.ice_getIdentity().Equals(Ice.Util.stringToIdentity("test")));
-            test(c2.ice_getIdentity().Equals(Ice.Util.stringToIdentity("noSuchIdentity")));
-            test(r.ice_getIdentity().Equals(Ice.Util.stringToIdentity("test")));
+            test(c1.ice_getIdentity().Equals(_communicator.stringToIdentity("test")));
+            test(c2.ice_getIdentity().Equals(_communicator.stringToIdentity("noSuchIdentity")));
+            test(r.ice_getIdentity().Equals(_communicator.stringToIdentity("test")));
 	    // We can't do the callbacks below in thread per connection mode.
-	    if(Ice.Util.getDefaultProperties().getPropertyAsInt("Ice.ThreadPerConnection") == 0)
+	    if(_communicator.getProperties().getPropertyAsInt("Ice.ThreadPerConnection") == 0)
 	    {
 		r.opVoid();
 		c1.opVoid();
@@ -291,10 +296,16 @@ public class TwowaysAMI
         }
         
         private Callback callback = new Callback();
+	private Ice.Communicator _communicator;
     }
     
     private class AMI_MyClass_opStructI : Test.AMI_MyClass_opStruct
     {
+	public AMI_MyClass_opStructI(Ice.Communicator comunicator)
+	{
+	    _communicator = comunicator;
+	}
+
         public override void ice_response(Test.Structure rso, Test.Structure so)
         {
             test(rso.p == null);
@@ -303,7 +314,7 @@ public class TwowaysAMI
             test(so.e == Test.MyEnum.enum3);
             test(so.s.s.Equals("a new string"));
 	    // We can't do the callbacks below in thread per connection mode.
-	    if(Ice.Util.getDefaultProperties().getPropertyAsInt("Ice.ThreadPerConnection") == 0)
+	    if(_communicator.getProperties().getPropertyAsInt("Ice.ThreadPerConnection") == 0)
 	    {
 		so.p.opVoid();
 	    }
@@ -321,6 +332,7 @@ public class TwowaysAMI
         }
         
         private Callback callback = new Callback();
+        private Ice.Communicator _communicator;
     }
     
     private class AMI_MyClass_opByteSI : Test.AMI_MyClass_opByteS
@@ -947,7 +959,8 @@ public class TwowaysAMI
         private Callback callback = new Callback();
     }
     
-    internal static void twowaysAMI(Ice.Communicator communicator, Test.MyClassPrx p)
+    internal static void twowaysAMI(Ice.Communicator communicator, 
+				    Ice.InitializationData initData, Test.MyClassPrx p)
     {
         {
 	    // Check that a call to a void operation raises TwowayOnlyException
@@ -1024,7 +1037,7 @@ public class TwowaysAMI
         }
         
         {
-            AMI_MyClass_opMyClassI cb = new AMI_MyClass_opMyClassI();
+            AMI_MyClass_opMyClassI cb = new AMI_MyClass_opMyClassI(communicator);
             p.opMyClass_async(cb, p);
             test(cb.check());
         }
@@ -1041,7 +1054,7 @@ public class TwowaysAMI
             si2.s = new Test.AnotherStruct();
             si2.s.s = "def";
             
-            AMI_MyClass_opStructI cb = new AMI_MyClass_opStructI();
+            AMI_MyClass_opStructI cb = new AMI_MyClass_opStructI(communicator);
             p.opStruct_async(cb, si1, si2);
             test(cb.check());
         }
@@ -1254,7 +1267,7 @@ public class TwowaysAMI
         }
         
         {
-            Ice.Context ctx = new Ice.Context();
+	    Ice.Context ctx = new Ice.Context();
             ctx["one"] = "ONE";
             ctx["two"] = "TWO";
             ctx["three"] = "THREE";
@@ -1270,7 +1283,7 @@ public class TwowaysAMI
                 p.opContext_async(cb, ctx);
                 test(cb.check());
             }
-            Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_newContext(ctx));
+            Test.MyClassPrx p2 = Test.MyClassPrxHelper.checkedCast(p.ice_context(ctx));
             test(p2.ice_getContext().Equals(ctx));
             {
                 AMI_MyClass_opContextEqualI cb = new AMI_MyClass_opContextEqualI(ctx);
@@ -1295,7 +1308,7 @@ public class TwowaysAMI
 		    test(cb.check());
 		}
 
-		p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_newContext(new Ice.Context()));
+		p2 = Test.MyClassPrxHelper.uncheckedCast(p.ice_context(new Ice.Context()));
 		{
 		    AMI_MyClass_opContextEqualI cb = new AMI_MyClass_opContextEqualI(new Ice.Context());
 		    p2.opContext_async(cb);
@@ -1318,7 +1331,7 @@ public class TwowaysAMI
 
 		communicator.setDefaultContext(dflt);
 		Test.MyClassPrx c = Test.MyClassPrxHelper.checkedCast(
-						communicator.stringToProxy("test:default -p 12345 -t 10000"));
+						communicator.stringToProxy("test:default -p 12010 -t 10000"));
 		{
 		    Ice.Context tmp = new Ice.Context();
 		    tmp["a"] = "b";
@@ -1328,7 +1341,7 @@ public class TwowaysAMI
 		}
 
 		dflt["a"] = "c";
-		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_newContext(dflt));
+		Test.MyClassPrx c2 = Test.MyClassPrxHelper.uncheckedCast(c.ice_context(dflt));
 		{
 		    Ice.Context tmp = new Ice.Context();
 		    tmp["a"] = "c";
@@ -1338,7 +1351,7 @@ public class TwowaysAMI
 		}
 
 		dflt.Clear();
-		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_newContext(dflt));
+		Test.MyClassPrx c3 = Test.MyClassPrxHelper.uncheckedCast(c2.ice_context(dflt));
 		{
 		    Ice.Context tmp = new Ice.Context();
 		    AMI_MyClass_opContextEqualI cb = new AMI_MyClass_opContextEqualI(tmp);
