@@ -1,6 +1,6 @@
 // **********************************************************************
 //
-// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
+// Copyright (c) 2003-2006 ZeroC, Inc. All rights reserved.
 //
 // This copy of Ice is licensed to you under the terms described in the
 // ICE_LICENSE file included in this distribution.
@@ -13,7 +13,7 @@ public class Server : Ice.Application
     {
 	args = communicator().getProperties().parseCommandLineOptions("TestAdapter", args);
         Ice.ObjectAdapter adapter = communicator().createObjectAdapter("TestAdapter");
-        adapter.add(new TestI(adapter), Ice.Util.stringToIdentity("test"));
+        adapter.add(new TestI(adapter), communicator().stringToIdentity("test"));
 	shutdownOnInterrupt();
 	try
 	{
@@ -30,6 +30,9 @@ public class Server : Ice.Application
     {
 	Server server = new Server();
 	int status = server.main(args);
-	System.Environment.Exit(status);
+        if(status != 0)
+        {
+            System.Environment.Exit(status);
+        }
     }
 }
